/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include "Classifier_Support.h"

uint8_t classify(double* X, Classifier_params_ptr Params)
{
        //Define result variables
        double learner_scores[NUM_LEARNERS]={0};
        double class_scores[NUM_CLASSES]={0};

        uint8_t prediction =0;
        uint8_t v,c;

        //STEP 1: Learner scores
		for(v=0;v<NUM_LEARNERS;v++)
		{
			vector_subtract_inplace(X,&(Params->Means[v*FEATURE_LENGTH]),FEATURE_LENGTH);
			vector_scale_inplace(X,&(Params->Std_Devs[v*FEATURE_LENGTH]),FEATURE_LENGTH);
			vector_dotproduct(X,&(Params->Weights[v*FEATURE_LENGTH]),&(learner_scores[v]),FEATURE_LENGTH);				
			learner_scores[v] += Params->Bias[v];
		}	

        //STEP 2: Class Scores
		for(c=0;c<NUM_CLASSES;c++)
		{
			vector_dotproduct(learner_scores,&(Params->Codmat[c*NUM_LEARNERS]),&(class_scores[c]),NUM_LEARNERS);
			class_scores[c] = hinge(class_scores[c]);
		}
		
		//Find prediction = max class scores
		prediction = max(class_scores,NUM_CLASSES);

        return prediction;
}
