/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <iostream>

#include "rndclab_body_subgraph.h"

#define NAME_LENGTH 20

rndclab_body_subgraph::rndclab_body_subgraph(char *in_file,
        char *out_file, int img_num, int *param) {
    int token_size;
    token_size = sizeof(Mat*);
    this->actor_count = ACTOR_COUNT;
    this->fifo_count = FIFO_COUNT;

    for (int i = 0; i<this->fifo_count; i++){
        fifos.push_back((lide_c_fifo_pointer)welt_c_fifo_new
        (BUFFER_CAPACITY,token_size,i));
    }

    this->in_file = in_file;
    this->out_file = out_file;
    this->img_num = img_num;

    /* Create and connect the actors. */
    /* Img source actor */
    actors.push_back(new rndclab_img_read(this->in_file, this->img_num,
                                                  this->fifos[FIFO_IMSRC_MC]));
    descriptors.push_back("actor img read");
    actors[ACTOR_IMG_SOURCE]->connect((welt_cpp_graph*) this);

    /* Motion Correction actor */
    actors.push_back(new rndclab_img_motioncorrection
    (this->fifos[FIFO_IMSRC_MC], this->fifos[FIFO_MC_F1]));
    descriptors.push_back("actor motion correction");
    actors[ACTOR_MOTION_CORRECTION]->connect((welt_cpp_graph*) this);

    /* Fork1 actor*/
    actors.push_back(new rndclab_img_fork(this->fifos[FIFO_MC_F1],
            this->fifos[FIFO_F1_PP],this->fifos[FIFO_F1_SE]));
    descriptors.push_back("actor img fork");
    actors[ACTOR_FORK]->connect((welt_cpp_graph*) this);

    /* Preprocessing actor*/
    actors.push_back(new rndclab_img_preprocess(this->fifos[FIFO_F1_PP],
            this->fifos[FIFO_PP_DT]));
    descriptors.push_back("actor img preprocessing");
    actors[ACTOR_PREPROCESSING]->connect((welt_cpp_graph*) this);

    /* Detection actor*/
    actors.push_back(new rndclab_img_detection(this->fifos[FIFO_PP_DT],
            this->fifos[FIFO_DT_SNK],this->fifos[FIFO_DT_SE], param));
    descriptors.push_back("actor img detection");
    actors[ACTOR_IMG_DETECTION]->connect((welt_cpp_graph*) this);

    /* Signal Extraction actor*/
    actors.push_back(new rndclab_signal_extraction
    (this->fifos[FIFO_F1_SE],this->fifos[FIFO_DT_SE],this->fifos[FIFO_SE_PD]));
    descriptors.push_back("actor signal extraction");
    actors[ACTOR_SIGNAL_EXTRACTION]->connect((welt_cpp_graph*) this);

    /* Sink actor */
    actors.push_back(new rndclab_img_write(this->fifos[FIFO_DT_SNK],
            this->out_file, img_num));
    descriptors.push_back("actor sink");
    actors[ACTOR_IMG_SINK]->connect((welt_cpp_graph*) this);

    /* Initialize source array and sink array */
//    this->source_array = (lide_c_actor_type **) lide_c_util_malloc(
//            this->fifo_count * sizeof(lide_c_actor_type *));
//
//    this->sink_array = (lide_c_actor_type **) lide_c_util_malloc(
//            this->fifo_count * sizeof(lide_c_actor_type *));

    /* Prediction actor*/
//    this->actors[ACTOR_PREDICTION] =
//            (lide_c_actor_type *) (lide_c_SVM_new(
//                    this->fifos[FIFO_SE_PD]));
//    lide_c_SVM_connect(this->actors[ACTOR_PREDICTION],
//                       (lide_c_graph_type *) context);
}

void rndclab_body_subgraph::scheduler() {
    /* ndsep body sub-graph scheduler */
    /* fire img_read actor first */
    welt_cpp_util_guarded_execution(actors[ACTOR_IMG_SOURCE],
            descriptors[ACTOR_IMG_SOURCE]);
    /* fire other actors then */
    boolean progress = FALSE;
    do {
        progress = 0;
        for (int i = ACTOR_MOTION_CORRECTION; i < actor_count; i++) {
            progress |=
                    welt_cpp_util_guarded_execution(actors[i], descriptors[i]);
        }
    } while (progress);

    /* simple scheduler */
//    welt_cpp_util_simple_scheduler(actors, actor_count, descriptors);
}

rndclab_body_subgraph::~rndclab_body_subgraph() {
    cout << "delete rndclab body subgraph" << endl;
}

void rndclab_body_subgraph_terminate(
        rndclab_body_subgraph *context) {
//    int i;
//    /* Terminate FIFO*/
//    for (i = 0; i < context->fifo_count; i++) {
//        lide_c_fifo_basic_free((lide_c_fifo_basic_pointer) context->fifos[i]);
//    }

    /* Terminate Actors*/
    rndclab_img_read_terminate((rndclab_img_read *)
            context->actors[ACTOR_IMG_SOURCE]);
    rndclab_img_motioncorrection_terminate(
            (rndclab_img_motioncorrection *)
                    context->actors[ACTOR_MOTION_CORRECTION]);
    rndclab_img_fork_terminate((rndclab_img_fork *)
            context->actors[ACTOR_FORK]);
    rndclab_img_preprocess_terminate((rndclab_img_preprocess *)
            context->actors[ACTOR_PREPROCESSING]);
    rndclab_img_detection_terminate((rndclab_img_detection *)
            context->actors[ACTOR_IMG_DETECTION]);
    rndclab_signal_extraction_terminate(
            (rndclab_signal_extraction *)
                    context->actors[ACTOR_SIGNAL_EXTRACTION]);
    rndclab_img_write_terminate(
            (rndclab_img_write *) context->actors[ACTOR_IMG_SINK]);

//    free(context->fifos);
//    free(context->actors);
//    free(context->descriptors);
    delete context;
    return;
}
