#ifndef _rndclab_body_subgraph_h
#define _rndclab_body_subgraph_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

extern "C" {
#include "lide_c_basic.h"
#include "lide_c_actor.h"
#include "lide_c_fifo.h"
//#include "lide_c_fifo_basic.h"
#include "welt_c_fifo.h"
#include "lide_c_graph.h"
#include "lide_c_util.h"
}
#include "welt_cpp_graph.h"
#include "welt_cpp_util.h"

#include "rndclab_img_read.h"
#include "rndclab_img_write.h"
#include "rndclab_img_detection.h"
#include "rndclab_img_motioncorrection.h"
#include "rndclab_signal_extraction.h"
#include "rndclab_img_fork.h"
#include "rndclab_img_preprocess.h"

#define BUFFER_CAPACITY 1024

/* An enumeration of the actors in this application. */
#define ACTOR_IMG_SOURCE 0
#define ACTOR_MOTION_CORRECTION 1
#define ACTOR_FORK         2
#define ACTOR_PREPROCESSING 3
#define ACTOR_IMG_DETECTION 4
#define ACTOR_SIGNAL_EXTRACTION 5
#define ACTOR_IMG_SINK     6
//#define ACTOR_PREDICTION   7

#define FIFO_IMSRC_MC      0
#define FIFO_MC_F1         1
#define FIFO_F1_PP         2
#define FIFO_F1_SE         3
#define FIFO_PP_DT         4
#define FIFO_SE_PD         5
#define FIFO_DT_SE         6
#define FIFO_DT_SNK        7

#define ACTOR_COUNT     7
#define FIFO_COUNT      8

/*******************************************************************************
 TYPE DEFINITIONS
 *******************************************************************************/
class rndclab_body_subgraph : public welt_cpp_graph{
public:
    rndclab_body_subgraph(char *in_file, char *out_file, int img_num,
            int *param);

    virtual ~rndclab_body_subgraph();

    void scheduler() override;

private:
    char *in_file;
    char *out_file;
    int img_num;
    int detectedResult;
};

/*******************************************************************************
 INTERFACE FUNCTIONS
 *******************************************************************************/

void rndclab_body_subgraph_terminate(
        rndclab_body_subgraph *context);

#endif
