/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <math.h>

#include "rndclab_img_detection.h"

#define MAX_FIFO_COUNT  3
#define PARAM_COUNT_DETECT 1
using namespace cv;
using namespace std;
extern int img_num;

rndclab_img_detection::rndclab_img_detection(
        lide_c_fifo_pointer img_in, lide_c_fifo_pointer img_out,
        lide_c_fifo_pointer det_out, int *param) {
    this->mode = RNDCLAB_IMG_DETECTION_MODE_PROCESS;

    /*img input - tokens are pointers to image data*/
    this->img_in = (welt_c_fifo_pointer) img_in;

    /*img output - tokens are pointers to image data*/
    this->img_out = (welt_c_fifo_pointer) img_out;

    this->det_out = (welt_c_fifo_pointer) det_out;
    this->last_keypoints = new std::vector<KeyPoint>();

    this->subgraph_count = 0;
    this->max_port_count = MAX_FIFO_COUNT;
    this->param = param;

    /* old way passing parameters */
//    this->param_length = (int *) lide_c_util_malloc(sizeof(int));
//    this->param_length[0] = 4;
//    this->max_param_count = PARAM_COUNT_DETECT;
//    welt_cpp_actor_init_vari_paramrefs(this,
//                                     this->param_length, PARAM_COUNT_DETECT,
//                                     this->param);
//    this->param_array = (int*)params_v[0];
}

rndclab_img_detection::~rndclab_img_detection() {
    cout << "delete img detection actor" << endl;
}

int rndclab_img_detection::enable() {
    boolean result = FALSE;

    switch (mode) {

        case RNDCLAB_IMG_DETECTION_MODE_INACTIVE:result = FALSE;
            break;
        case RNDCLAB_IMG_DETECTION_MODE_PROCESS:
            result = (welt_c_fifo_population(img_out) <
                      welt_c_fifo_capacity(img_out)) &&
                    (welt_c_fifo_population(det_out) <
                    welt_c_fifo_capacity(det_out)) &&
                    (welt_c_fifo_population(img_in)>=1);
            break;
        default:result = FALSE;
            break;
    }
    return result;
}

void rndclab_img_detection::invoke() {
    switch (mode) {

        case RNDCLAB_IMG_DETECTION_MODE_INACTIVE:
            mode = RNDCLAB_IMG_DETECTION_MODE_INACTIVE;

            break;
        case RNDCLAB_IMG_DETECTION_MODE_PROCESS: {
            cout << "params in detection actor: " << param[0] << param[1] <<
            param[2] << param[3] << endl;
            clock_t begin = clock();
            std::vector<KeyPoint> keypoints; /* To get detected points */
            std::vector<float> roi_sig(200, 0);
            Mat *Inewimg;

            /* Get input */
            welt_c_fifo_read(img_in, &(Inewimg));
            cv::Mat invimg;
            bool replaced = false;
            /* When input is not a gray image */
            /* cvCvtColor(Inewimg,im_gray,CV_RGB2GRAY); */
            cv::Mat newimg = *Inewimg;

            Ptr<SimpleBlobDetector> detector;
            /* When the blob(neuron) in input image has higher intensity than
             * background */
            invimg = cv::Scalar::all(255) - newimg;

            /* In initialization mode */
            if (param[0] == 0) {
                /*Set parameters for the blob detector*/
                SimpleBlobDetector::Params params;

                /* Key parameters for offline version (manually tuned) */
                params.minThreshold = 0;
                params.thresholdStep = param[1];
                params.maxThreshold = 255;

                params.filterByArea = true;
                params.minArea = param[2];
                params.maxArea = param[3];

                /* Users can use the below parameters as well after setting
                 * 'True'. For detail information about the parameters,
                 * please refer to
                 * https://www.learnopencv.com/blob-detection-using-opencv-python-c
                 * */

                /* Filter by Circularity */
                params.filterByCircularity = false;
                params.minCircularity = 0.00;

                /* Filter by Convexity */
                params.filterByConvexity = false;
                params.minConvexity = 0.00;

                /* Filter by Inertia */
                params.filterByInertia = false;
                params.minInertiaRatio = 0.00;

                /* Define a detector */
                detector = SimpleBlobDetector::create(params);

                /* Detect */
                detector->detect(invimg, keypoints);

                /* When it detects the same blob as in the previous frame, erase
                 * it or replace it */
                int u, k, c, r;
                for (u = 0; u < last_keypoints->size(); u++) {
                    replaced = false;
                    for (k = 0; k < keypoints.size(); k++) {

                        if ((int(pow((*(last_keypoints))[u].pt.x
                                     - keypoints[k].pt.x, 2.0)) + int(pow(
                                (*(last_keypoints))[u].pt.y
                                - keypoints[k].pt.y, 2.0)) < int(pow(
                                (*(last_keypoints))[u].size / 2
                                + keypoints[k].size / 2, 2.0)))) {

                            if ((replaced = false) && (int(pow(
                                    (*(last_keypoints))[u].pt.x
                                    - keypoints[k].pt.x, 2.0))
                                                       + int(pow(
                                    (*(last_keypoints))[u].pt.y
                                    - keypoints[k].pt.y, 2.0)))
                                                      < int(pow(
                                    (*(last_keypoints))[u].size
                                    / 4 + keypoints[k].size / 4,
                                    2.0))) {

                                (*(last_keypoints))[u] = keypoints[k];
                                replaced = true;

                            }

                            keypoints.erase(keypoints.begin() + k);
                            k = k - 1;
                        }

                    }

                }

                /* Combine to last detected neurons */
                (*(last_keypoints))
                        .insert((*(last_keypoints)).end(),
                                keypoints.begin(), keypoints.end());

            }

            Mat im_with_keypoints;

            /* To draw the output of the detection */
            drawKeypoints(newimg, (*(last_keypoints)),
                          im_with_keypoints, Scalar(0, 255, 0),
                          DrawMatchesFlags::DRAW_RICH_KEYPOINTS);

            /* Write the output image to fifo (The detected key points will be
             * imported in the next actor as a global variable.) */

            /* write number of detected neurons to param to be read by
             * set_param actor */
            param[4] = last_keypoints->size() - 1;
            std::cout << "Number of detected neurons :" << param[4]
                      << std::endl;

            inner_image = im_with_keypoints;
            Mat *img_send = &inner_image;
            welt_c_fifo_write(img_out,&img_send);

            //the vector to be transmitted couldn't be empty
            if ((last_keypoints)->size() == 0) {
                KeyPoint keyp;
                keyp.pt.x = 0;
                keyp.pt.y = 0;
                keyp.size = 0;
                (last_keypoints)->push_back(keyp);
            }

            welt_c_fifo_write(det_out, &last_keypoints);
            clock_t end = clock();
            double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
            std::cout << "Detection execution time :" << elapsed_secs
                      << std::endl;
            mode = RNDCLAB_IMG_DETECTION_MODE_PROCESS;
            break;
        }
        default:mode = RNDCLAB_IMG_DETECTION_MODE_INACTIVE;
            break;
    }
}

void rndclab_img_detection::reset() {
    mode = RNDCLAB_IMG_DETECTION_MODE_PROCESS;
}

void rndclab_img_detection::connect(welt_cpp_graph *graph) {
//    int port_index;
//    int direction;
//    /* input 1*/
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 1;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//
//    /* output 2*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 2;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
}

void rndclab_img_detection_terminate(
        rndclab_img_detection *context) {
    delete context;
}
