#ifndef _rndclab_img_detection_h
#define _rndclab_img_detection_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

extern "C" {
#include "lide_c_util.h"
#include "lide_c_actor.h"
#include "lide_c_fifo.h"
//#include "lide_c_fifo_basic.h"
#include "welt_c_fifo.h"
}

#include "welt_cpp_actor.h"
#include "welt_cpp_graph.h"
#include "welt_cpp_actor.h"

#include <opencv2/opencv.hpp>
#include <opencv2/core.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc.hpp>

#define RNDCLAB_IMG_DETECTION_MODE_INACTIVE 1
#define RNDCLAB_IMG_DETECTION_MODE_PROCESS 2

using namespace cv;

class rndclab_img_detection : public welt_cpp_actor{
public:
    rndclab_img_detection(
            lide_c_fifo_pointer img_in, lide_c_fifo_pointer img_out,
            lide_c_fifo_pointer det_out, int *param);

    int enable() override;

    void invoke() override;

    void reset() override;

    void connect(welt_cpp_graph *graph) override;

    virtual ~rndclab_img_detection();

private:
    /*Actor interface ports.*/
    welt_c_fifo_pointer img_in;
    welt_c_fifo_pointer img_out;
    welt_c_fifo_pointer det_out;
    int *param;
    std::vector<KeyPoint> *last_keypoints;

    /* Inner state variable prevent from being auto released */
    Mat inner_image;

    int* param_array;
};

void rndclab_img_detection_terminate(
        rndclab_img_detection *context);

#endif

