/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <iostream>

#include "rndclab_img_fork.h"

#define MAX_FIFO_COUNT  3

#include <iostream>


using namespace cv;
using namespace std;

rndclab_img_fork::rndclab_img_fork(lide_c_fifo_pointer in,
                                                   lide_c_fifo_pointer out1,
                                                   lide_c_fifo_pointer out2) {
    /* Set actor mode to ACTIVE */
    this->mode = RNDCLAB_FORK1_MODE_ACTIVE;
    /*img input - tokens are pointers to image data*/
    this->in = (welt_c_fifo_pointer) in;
    /*img output - tokens are pointers to image data*/
    this->out1 = (welt_c_fifo_pointer) out1;
    this->out2 = (welt_c_fifo_pointer) out2;
    this->subgraph_count = 0;
    this->max_port_count = MAX_FIFO_COUNT;
}

rndclab_img_fork::~rndclab_img_fork() {

}

int rndclab_img_fork::enable() {
    boolean retval = FALSE;
    switch (mode) {
        case RNDCLAB_FORK1_MODE_INACTIVE:retval = FALSE;
            break;
        case RNDCLAB_FORK1_MODE_ACTIVE:
            retval = (welt_c_fifo_population(in) >= 1)
                     && (welt_c_fifo_population(out1) <
                         welt_c_fifo_capacity(out1)) &&
                         (welt_c_fifo_population(out2)
                         <welt_c_fifo_capacity(out2));
            break;
        default:retval = FALSE;
            break;
    }
    return retval;
}

void rndclab_img_fork::invoke() {
    switch (mode) {
        /* Inactive actor state */
        case RNDCLAB_FORK1_MODE_INACTIVE:mode = RNDCLAB_FORK1_MODE_INACTIVE;
            break;
        /* Active actor state */
        case RNDCLAB_FORK1_MODE_ACTIVE: {
            Mat *src_img = nullptr;
//            Mat *src_img_1 = nullptr;
//            Mat *src_img_2 = nullptr;
            /* Get input */
            welt_c_fifo_read(in, &src_img);
//            src_img_1 = src_img;
////            FIXME: check whether img is cloned
//            src_img_2 = src_img;
//            FIXME: img is NOT cloned
            welt_c_fifo_write(out1, &src_img);
            welt_c_fifo_write(out2, &src_img);

            mode = RNDCLAB_FORK1_MODE_ACTIVE;
            break;
        }
        default:mode = RNDCLAB_FORK1_MODE_INACTIVE;
            break;
    }
}

void rndclab_img_fork::reset() {
    mode = RNDCLAB_FORK1_MODE_ACTIVE;
}

void rndclab_img_fork::connect(welt_cpp_graph *graph) {
//    int port_index;
//    int direction;
//    /* input 1*/
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 1;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 2;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//    return;
}

void rndclab_img_fork_terminate(rndclab_img_fork *context) {
    delete context;
}
