/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <cassert>
#include <cmath>
#include <fstream>
#include <string>
#include <ctime>
#include <algorithm>

#include "rndclab_img_motioncorrection.h"

#define MAX_FIFO_COUNT  2


int frame_num = 1;
using namespace cv;
using namespace std;
extern int img_num;
int num = 0;


//std::list<double> sta;

//Mat cur_grey_ori;
//Mat cur;
//Mat cur_grey;
//Mat prev;
//Mat prev_grey;
//Mat cur2;
//Mat last_T;

Mat GetGradient(Mat src_gray) {
    Mat grad_x, grad_y;
    Mat abs_grad_x, abs_grad_y;

    int scale = 1;
    int delta = 0;
    int ddepth = CV_32FC1;;

    /* Calculate the x and y gradients using Sobel operator */

    Sobel(src_gray, grad_x, ddepth, 1, 0, 3, scale, delta, BORDER_DEFAULT);
    convertScaleAbs(grad_x, abs_grad_x);

    Sobel(src_gray, grad_y, ddepth, 0, 1, 3, scale, delta, BORDER_DEFAULT);
    convertScaleAbs(grad_y, abs_grad_y);

    /* Combine the two gradients */
    Mat grad;
    addWeighted(abs_grad_x, 0.5, abs_grad_y, 0.5, 0, grad);

    return grad;

}

// A structure to represent a queue
struct CorrQueue {
    int front, rear, size;
    unsigned capacity;
    double *array;
};

// function to create a queue of given capacity.
// It initializes size of queue as 0
struct CorrQueue *createQueue(unsigned capacity) {
    struct CorrQueue *queue =
            (struct CorrQueue *) lide_c_util_malloc(sizeof(struct CorrQueue));
    queue->capacity = capacity;
    queue->front = queue->size = 0;
    queue->rear = capacity - 1;  // This is important, see the enqueue
    queue->array =
            (double *) lide_c_util_malloc(queue->capacity * sizeof(double));
    return queue;
}

// Queue is full when size becomes equal to the capacity
int isFull(struct CorrQueue *queue) { return (queue->size == queue->capacity); }

// Queue is empty when size is 0
int isEmpty(struct CorrQueue *queue) { return (queue->size == 0); }

// Functions to remove an item from queue.
// They changes front, rear, and size
double dequeue(struct CorrQueue *queue) {
    if (isEmpty(queue)) {
        printf("queue is empty\n");
        return 0;
    }
    double item = queue->array[queue->front];
    queue->front = (queue->front + 1) % queue->capacity;
    queue->size = queue->size - 1;
    return item;
}

double popBack(struct CorrQueue *queue) {
    if (isEmpty(queue)) {
        printf("queue is empty\n");
        return 0;
    }
    double item = queue->array[queue->rear];
    queue->rear = (queue->rear - 1) % queue->capacity;
    queue->size = queue->size - 1;
    return item;
}

// Function to add an item to the queue.
// It changes rear and size
void enqueue(struct CorrQueue *queue, double item) {
    if (isFull(queue)) {
        printf("MC: queue popd %f\n", dequeue(queue));
    }
    queue->rear = (queue->rear + 1) % queue->capacity;
    queue->array[queue->rear] = item;
    queue->size = queue->size + 1;
    printf("MC: %f enqueued to queue\n", item);
}

// Function to get front of queue
double front(struct CorrQueue *queue) {
    if (isEmpty(queue))
        return 0;
    return queue->array[queue->front];
}

// Function to get rear of queue
double rear(struct CorrQueue *queue) {
    if (isEmpty(queue))
        return 0;
    return queue->array[queue->rear];
}

void showQueue(struct CorrQueue *queue) {
    if (isEmpty(queue)) {
        printf("queue is empty\n");
        return;
    } else {
        for (int i = queue->front; i < (queue->front + queue->size); i++) {
            printf("%f\t", queue->array[i % queue->capacity]);
        }
        printf("\n");
    }
}

void meanQueue(struct CorrQueue *queue, double *mean, double *summean) {
    double sum = 0;
    *summean = 0;
    for (int i = queue->front; i < (queue->front + queue->size); i++) {
        sum += queue->array[i % queue->capacity];
    }

    *mean = (double) sum / queue->size;
    sum = 0;
    for (int i = queue->front; i < (queue->front + queue->size); i++) {
        //        printf("%f\n ",*it - *mean);
        sum += pow((queue->array[i % queue->capacity] - *mean), 2);
        //        printf("%f\n ",sum);
        *summean = sqrt((double) sum / (queue->size - 1));

    }
}

static void image_jacobian_homo_ECC(const Mat &src1, const Mat &src2,
        const Mat &src3, const Mat &src4,
        const Mat &src5, Mat &dst) {

    CV_Assert(src1.size() == src2.size());
    CV_Assert(src1.size() == src3.size());
    CV_Assert(src1.size() == src4.size());

    CV_Assert(src1.rows == dst.rows);
    CV_Assert(dst.cols == (src1.cols * 8));
    CV_Assert(dst.type() == CV_32FC1);

    CV_Assert(src5.isContinuous());

    const float *hptr = src5.ptr<float>(0);

    const float h0_ = hptr[0];
    const float h1_ = hptr[3];
    const float h2_ = hptr[6];
    const float h3_ = hptr[1];
    const float h4_ = hptr[4];
    const float h5_ = hptr[7];
    const float h6_ = hptr[2];
    const float h7_ = hptr[5];

    const int w = src1.cols;


    //create denominator for all points as a block
    Mat den_ = src3 * h2_ + src4 * h5_
            + 1.0;//check the time of this! otherwise use addWeighted

    //create projected points
    Mat hatX_ = -src3 * h0_ - src4 * h3_ - h6_;
    divide(hatX_, den_, hatX_);
    Mat hatY_ = -src3 * h1_ - src4 * h4_ - h7_;
    divide(hatY_, den_, hatY_);


    //instead of dividing each block with den,
    //just pre-devide the block of gradients (it's more efficient)

    Mat src1Divided_;
    Mat src2Divided_;

    divide(src1, den_, src1Divided_);
    divide(src2, den_, src2Divided_);


    //compute Jacobian blocks (8 blocks)

    dst.colRange(0, w) = src1Divided_.mul(src3);//1

    dst.colRange(w, 2 * w) = src2Divided_.mul(src3);//2

    Mat temp_ = (hatX_.mul(src1Divided_) + hatY_.mul(src2Divided_));
    dst.colRange(2 * w, 3 * w) = temp_.mul(src3);//3

    hatX_.release();
    hatY_.release();

    dst.colRange(3 * w, 4 * w) = src1Divided_.mul(src4);//4

    dst.colRange(4 * w, 5 * w) = src2Divided_.mul(src4);//5

    dst.colRange(5 * w, 6 * w) = temp_.mul(src4);//6

    src1Divided_.copyTo(dst.colRange(6 * w, 7 * w));//7

    src2Divided_.copyTo(dst.colRange(7 * w, 8 * w));//8
}

static void image_jacobian_euclidean_ECC(const Mat &src1, const Mat &src2,
        const Mat &src3, const Mat &src4,
        const Mat &src5, Mat &dst) {

    CV_Assert(src1.size() == src2.size());
    CV_Assert(src1.size() == src3.size());
    CV_Assert(src1.size() == src4.size());

    CV_Assert(src1.rows == dst.rows);
    CV_Assert(dst.cols == (src1.cols * 3));
    CV_Assert(dst.type() == CV_32FC1);

    CV_Assert(src5.isContinuous());

    const float *hptr = src5.ptr<float>(0);

    const float h0 = hptr[0];//cos(theta)
    const float h1 = hptr[3];//sin(theta)

    const int w = src1.cols;

    //create -sin(theta)*X -cos(theta)*Y for all points as a block -> hatX
    Mat hatX = -(src3 * h1) - (src4 * h0);

    //create cos(theta)*X -sin(theta)*Y for all points as a block -> hatY
    Mat hatY = (src3 * h0) - (src4 * h1);


    //compute Jacobian blocks (3 blocks)
    dst.colRange(0, w) = (src1.mul(hatX)) + (src2.mul(hatY));//1

    src1.copyTo(dst.colRange(w, 2 * w));//2
    src2.copyTo(dst.colRange(2 * w, 3 * w));//3
}

static void image_jacobian_affine_ECC(const Mat &src1, const Mat &src2,
        const Mat &src3, const Mat &src4,
        Mat &dst) {

    CV_Assert(src1.size() == src2.size());
    CV_Assert(src1.size() == src3.size());
    CV_Assert(src1.size() == src4.size());

    CV_Assert(src1.rows == dst.rows);
    CV_Assert(dst.cols == (6 * src1.cols));

    CV_Assert(dst.type() == CV_32FC1);

    const int w = src1.cols;

    //compute Jacobian blocks (6 blocks)

    dst.colRange(0, w) = src1.mul(src3);//1
    dst.colRange(w, 2 * w) = src2.mul(src3);//2
    dst.colRange(2 * w, 3 * w) = src1.mul(src4);//3
    dst.colRange(3 * w, 4 * w) = src2.mul(src4);//4
    src1.copyTo(dst.colRange(4 * w, 5 * w));//5
    src2.copyTo(dst.colRange(5 * w, 6 * w));//6
}

static void image_jacobian_translation_ECC(const Mat &src1, const Mat &src2,
        Mat &dst) {

    CV_Assert(src1.size() == src2.size());

    CV_Assert(src1.rows == dst.rows);
    CV_Assert(dst.cols == (src1.cols * 2));
    CV_Assert(dst.type() == CV_32FC1);

    const int w = src1.cols;

    //compute Jacobian blocks (2 blocks)
    src1.copyTo(dst.colRange(0, w));
    src2.copyTo(dst.colRange(w, 2 * w));
}

static void project_onto_jacobian_ECC(const Mat &src1, const Mat &src2,
        Mat &dst) {
    /* this functions is used for two types of projections. If src1.cols ==src.cols
    it does a blockwise multiplication (like in the outer product of vectors)
    of the blocks in matrices src1 and src2 and dst
    has size (number_of_blcks x number_of_blocks), otherwise dst is a vector of size
    (number_of_blocks x 1) since src2 is "multiplied"(dot) with each block of src1.

    The number_of_blocks is equal to the number of parameters we are lloking for
    (i.e. rtanslation:2, euclidean: 3, affine: 6, homography: 8)

    */
    CV_Assert(src1.rows == src2.rows);
    CV_Assert((src1.cols % src2.cols) == 0);
    int w;

    float *dstPtr = dst.ptr<float>(0);

    if (src1.cols != src2.cols) {//dst.cols==1
        w = src2.cols;
        for (int i = 0; i < dst.rows; i++) {
            dstPtr[i] = (float) src2.dot(src1.colRange(i * w, (i + 1) * w));
        }
    } else {
        CV_Assert(dst.cols == dst.rows); //dst is square (and symmetric)
        w = src2.cols / dst.cols;
        Mat mat;
        for (int i = 0; i < dst.rows; i++) {

            mat = Mat(src1.colRange(i * w, (i + 1) * w));
            dstPtr[i * (dst.rows + 1)] =
                    (float) pow(norm(mat), 2); //diagonal elements

            for (int j = i + 1; j < dst.cols; j++) { //j starts from i+1
                dstPtr[i * dst.cols + j] =
                        (float) mat.dot(src2.colRange(j * w, (j + 1) * w));
                dstPtr[j * dst.cols + i] =
                        dstPtr[i * dst.cols + j]; //due to symmetry
            }
        }
    }
}

static void update_warping_matrix_ECC(Mat &map_matrix, const Mat &update,
        const int motionType) {
    CV_Assert (map_matrix.type() == CV_32FC1);
    CV_Assert (update.type() == CV_32FC1);

    CV_Assert (
            motionType == MOTION_TRANSLATION || motionType == MOTION_EUCLIDEAN
                    ||
                            motionType == MOTION_AFFINE
                    || motionType == MOTION_HOMOGRAPHY);

    if (motionType == MOTION_HOMOGRAPHY)
        CV_Assert (map_matrix.rows == 3 && update.rows == 8);
    else if (motionType == MOTION_AFFINE)
        CV_Assert(map_matrix.rows == 2 && update.rows == 6);
    else if (motionType == MOTION_EUCLIDEAN)
        CV_Assert (map_matrix.rows == 2 && update.rows == 3);
    else CV_Assert (map_matrix.rows == 2 && update.rows == 2);

    CV_Assert (update.cols == 1);

    CV_Assert(map_matrix.isContinuous());
    CV_Assert(update.isContinuous());

    float *mapPtr = map_matrix.ptr<float>(0);
    const float *updatePtr = update.ptr<float>(0);

    if (motionType == MOTION_TRANSLATION) {
        mapPtr[2] += updatePtr[0];
        mapPtr[5] += updatePtr[1];
    }
    if (motionType == MOTION_AFFINE) {
        mapPtr[0] += updatePtr[0];
        mapPtr[3] += updatePtr[1];
        mapPtr[1] += updatePtr[2];
        mapPtr[4] += updatePtr[3];
        mapPtr[2] += updatePtr[4];
        mapPtr[5] += updatePtr[5];
    }
    if (motionType == MOTION_HOMOGRAPHY) {
        mapPtr[0] += updatePtr[0];
        mapPtr[3] += updatePtr[1];
        mapPtr[6] += updatePtr[2];
        mapPtr[1] += updatePtr[3];
        mapPtr[4] += updatePtr[4];
        mapPtr[7] += updatePtr[5];
        mapPtr[2] += updatePtr[6];
        mapPtr[5] += updatePtr[7];
    }
    if (motionType == MOTION_EUCLIDEAN) {
        double new_theta = updatePtr[0];
        new_theta += asin(mapPtr[3]);

        mapPtr[2] += updatePtr[1];
        mapPtr[5] += updatePtr[2];
        mapPtr[0] = mapPtr[4] = (float) cos(new_theta);
        mapPtr[3] = (float) sin(new_theta);
        mapPtr[1] = -mapPtr[3];
    }
}

double findTransformECC_J(InputArray templateImage,
        InputArray inputImage,
        InputOutputArray warpMatrix,
        int motionType,
        TermCriteria criteria,
        InputArray inputMask) {

    Mat src = templateImage.getMat();//template iamge
    Mat dst = inputImage.getMat(); //input image (to be warped)
    Mat map = warpMatrix.getMat(); //warp (transformation)

    CV_Assert(!src.empty());
    CV_Assert(!dst.empty());

    // If the user passed an un-initialized warpMatrix, initialize to identity
    if (map.empty()) {
        int rowCount = 2;
        if (motionType == MOTION_HOMOGRAPHY)
            rowCount = 3;

        warpMatrix.create(rowCount, 3, CV_32FC1);
        map = warpMatrix.getMat();
        map = Mat::eye(rowCount, 3, CV_32F);
    }

    if (!(src.type() == dst.type()))
        CV_Error(Error::StsUnmatchedFormats,
                "Both input images must have the same data type");

    //accept only 1-channel images
    if (src.type() != CV_8UC1 && src.type() != CV_32FC1)
        CV_Error(Error::StsUnsupportedFormat,
                "Images must have 8uC1 or 32fC1 type");

    if (map.type() != CV_32FC1)
        CV_Error(Error::StsUnsupportedFormat,
                "warpMatrix must be single-channel floating-point matrix");

    CV_Assert (map.cols == 3);
    CV_Assert (map.rows == 2 || map.rows == 3);

    CV_Assert (motionType == MOTION_AFFINE || motionType == MOTION_HOMOGRAPHY ||
            motionType == MOTION_EUCLIDEAN || motionType == MOTION_TRANSLATION);

    if (motionType == MOTION_HOMOGRAPHY) {
        CV_Assert (map.rows == 3);
    }

    CV_Assert (criteria.type & TermCriteria::COUNT
            || criteria.type & TermCriteria::EPS);
    const int numberOfIterations =
            (criteria.type & TermCriteria::COUNT) ? criteria.maxCount : 200;
    const double termination_eps =
            (criteria.type & TermCriteria::EPS) ? criteria.epsilon : -1;

    int paramTemp = 6;//default: affine
    switch (motionType) {
    case MOTION_TRANSLATION:paramTemp = 2;
        break;
    case MOTION_EUCLIDEAN:paramTemp = 3;
        break;
    case MOTION_HOMOGRAPHY:paramTemp = 8;
        break;
    }

    const int numberOfParameters = paramTemp;

    const int ws = src.cols;
    const int hs = src.rows;
    const int wd = dst.cols;
    const int hd = dst.rows;

    Mat Xcoord = Mat(1, ws, CV_32F);
    Mat Ycoord = Mat(hs, 1, CV_32F);
    Mat Xgrid = Mat(hs, ws, CV_32F);
    Mat Ygrid = Mat(hs, ws, CV_32F);

    float *XcoPtr = Xcoord.ptr<float>(0);
    float *YcoPtr = Ycoord.ptr<float>(0);
    int j;
    for (j = 0; j < ws; j++)
        XcoPtr[j] = (float) j;
    for (j = 0; j < hs; j++)
        YcoPtr[j] = (float) j;

    repeat(Xcoord, hs, 1, Xgrid);
    repeat(Ycoord, 1, ws, Ygrid);

    Xcoord.release();
    Ycoord.release();

    Mat templateZM = Mat(hs, ws,
            CV_32F);// to store the (smoothed)zero-mean version of template
    Mat templateFloat = Mat(hs, ws, CV_32F);// to store the (smoothed) template
    Mat imageFloat = Mat(hd, wd, CV_32F);// to store the (smoothed) input image
    Mat imageWarped =
            Mat(hs, ws, CV_32F);// to store the warped zero-mean input image
    Mat imageMask = Mat(hs, ws, CV_8U); //to store the final mask

    Mat inputMaskMat = inputMask.getMat();
    //to use it for mask warping
    Mat preMask;
    if (inputMask.empty())
        preMask = Mat::ones(hd, wd, CV_8U);
    else
        threshold(inputMask, preMask, 0, 1, THRESH_BINARY);

    //gaussian filtering is optional
    src.convertTo(templateFloat, templateFloat.type());
    GaussianBlur(templateFloat, templateFloat, Size(5, 5), 0, 0);

    Mat preMaskFloat;
    preMask.convertTo(preMaskFloat, CV_32F);
    GaussianBlur(preMaskFloat, preMaskFloat, Size(5, 5), 0, 0);
    // Change threshold.
    preMaskFloat *= (0.5 / 0.95);
    // Rounding conversion.
    preMaskFloat.convertTo(preMask, preMask.type());
    preMask.convertTo(preMaskFloat, preMaskFloat.type());

    dst.convertTo(imageFloat, imageFloat.type());
    GaussianBlur(imageFloat, imageFloat, Size(5, 5), 0, 0);

    // needed matrices for gradients and warped gradients
    Mat gradientX = Mat::zeros(hd, wd, CV_32FC1);
    Mat gradientY = Mat::zeros(hd, wd, CV_32FC1);
    Mat gradientXWarped = Mat(hs, ws, CV_32FC1);
    Mat gradientYWarped = Mat(hs, ws, CV_32FC1);


    // calculate first order image derivatives
    Matx13f dx(-0.5f, 0.0f, 0.5f);

    filter2D(imageFloat, gradientX, -1, dx);
    filter2D(imageFloat, gradientY, -1, dx.t());

    gradientX = gradientX.mul(preMaskFloat);
    gradientY = gradientY.mul(preMaskFloat);

    // matrices needed for solving linear equation system for maximizing ECC
    Mat jacobian = Mat(hs, ws * numberOfParameters, CV_32F);
    Mat hessian = Mat(numberOfParameters, numberOfParameters, CV_32F);
    Mat hessianInv = Mat(numberOfParameters, numberOfParameters, CV_32F);
    Mat imageProjection = Mat(numberOfParameters, 1, CV_32F);
    Mat templateProjection = Mat(numberOfParameters, 1, CV_32F);
    Mat imageProjectionHessian = Mat(numberOfParameters, 1, CV_32F);
    Mat errorProjection = Mat(numberOfParameters, 1, CV_32F);

    Mat deltaP = Mat(numberOfParameters, 1,
            CV_32F);//transformation parameter correction
    Mat error = Mat(hs, ws, CV_32F);//error as 2D matrix

    const int imageFlags = INTER_LINEAR + WARP_INVERSE_MAP;
    const int maskFlags = INTER_NEAREST + WARP_INVERSE_MAP;


    // iteratively update map_matrix
    double rho = -1;
    double last_rho = -termination_eps;
    for (int i = 1;
         (i <= numberOfIterations) && (fabs(rho - last_rho) >= termination_eps);
         i++) {

        // warp-back portion of the inputImage and gradients to the coordinate space of the templateImage
        if (motionType != MOTION_HOMOGRAPHY) {
            warpAffine(imageFloat, imageWarped, map, imageWarped.size(),
                    imageFlags);
            warpAffine(gradientX, gradientXWarped, map, gradientXWarped.size(),
                    imageFlags);
            warpAffine(gradientY, gradientYWarped, map, gradientYWarped.size(),
                    imageFlags);
            warpAffine(preMask, imageMask, map, imageMask.size(), maskFlags);
        } else {
            warpPerspective(imageFloat, imageWarped, map, imageWarped.size(),
                    imageFlags);
            warpPerspective(gradientX, gradientXWarped, map,
                    gradientXWarped.size(), imageFlags);
            warpPerspective(gradientY, gradientYWarped, map,
                    gradientYWarped.size(), imageFlags);
            warpPerspective(preMask, imageMask, map, imageMask.size(),
                    maskFlags);
        }

        Scalar imgMean, imgStd, tmpMean, tmpStd;
        meanStdDev(imageWarped, imgMean, imgStd, imageMask);
        meanStdDev(templateFloat, tmpMean, tmpStd, imageMask);

        subtract(imageWarped, imgMean, imageWarped, imageMask);//zero-mean input
        templateZM =
                Mat::zeros(templateZM.rows, templateZM.cols, templateZM.type());
        subtract(templateFloat, tmpMean, templateZM,
                imageMask);//zero-mean template

        const double tmpNorm = std::sqrt(
                countNonZero(imageMask) * (tmpStd.val[0]) * (tmpStd.val[0]));
        const double imgNorm = std::sqrt(
                countNonZero(imageMask) * (imgStd.val[0]) * (imgStd.val[0]));

        // calculate jacobian of image wrt parameters
        switch (motionType) {
        case MOTION_AFFINE:
            image_jacobian_affine_ECC(gradientXWarped, gradientYWarped, Xgrid,
                    Ygrid, jacobian);
            break;
        case MOTION_HOMOGRAPHY:
            image_jacobian_homo_ECC(gradientXWarped, gradientYWarped, Xgrid,
                    Ygrid, map, jacobian);
            break;
        case MOTION_TRANSLATION:
            image_jacobian_translation_ECC(gradientXWarped, gradientYWarped,
                    jacobian);
            break;
        case MOTION_EUCLIDEAN:
            image_jacobian_euclidean_ECC(gradientXWarped, gradientYWarped,
                    Xgrid, Ygrid, map, jacobian);
            break;
        }

        // calculate Hessian and its inverse
        project_onto_jacobian_ECC(jacobian, jacobian, hessian);

        hessianInv = hessian.inv();

        const double correlation = templateZM.dot(imageWarped);

        // calculate enhanced correlation coefficiont (ECC)->rho
        last_rho = rho;
        rho = correlation / (imgNorm * tmpNorm);
        if (cvIsNaN(rho)) {
//            CV_Error(Error::StsNoConv, "NaN encountered.");
        }

        // project images into jacobian
        project_onto_jacobian_ECC(jacobian, imageWarped, imageProjection);
        project_onto_jacobian_ECC(jacobian, templateZM, templateProjection);


        // calculate the parameter lambda to account for illumination variation
        imageProjectionHessian = hessianInv * imageProjection;
        const double lambda_n = (imgNorm * imgNorm)
                - imageProjection.dot(imageProjectionHessian);
        const double lambda_d =
                correlation - templateProjection.dot(imageProjectionHessian);
        if (lambda_d <= 0.0) {
            rho = -1;
//            CV_Error(Error::StsNoConv, "The algorithm stopped before its convergence. The correlation is going to be minimized. Images may be uncorrelated or non-overlapped");

        }
        const double lambda = (lambda_n / lambda_d);

        // estimate the update step delta_p
        error = lambda * templateZM - imageWarped;
        project_onto_jacobian_ECC(jacobian, error, errorProjection);
        deltaP = hessianInv * errorProjection;

        // update warping matrix
        update_warping_matrix_ECC(map, deltaP, motionType);

    }

    // return final correlation coefficient
    return rho;
}

rndclab_img_motioncorrection::rndclab_img_motioncorrection(
        lide_c_fifo_pointer in, lide_c_fifo_pointer out) {
    this->mode = RNDCLAB_IMG_MOTIONCORRECTION_MODE_PROCESS;
//    this->prev_img = (welt_c_fifo_pointer) in;
    /*img input - tokens are pointers to image data*/
    this->in = (welt_c_fifo_pointer) in;
    /*img output - tokens are pointers to image data*/
    this->out = (welt_c_fifo_pointer) out;
    /*Frame number */
    this->frame_num = 1;
    this->subgraph_count = 0;
    this->max_port_count = MAX_FIFO_COUNT;

    this->queue = createQueue(100);
}

int rndclab_img_motioncorrection::enable() {
    boolean result = FALSE;

    switch (mode) {
        case RNDCLAB_IMG_MOTIONCORRECTION_MODE_INACTIVE:result = FALSE;
            break;
        case RNDCLAB_IMG_MOTIONCORRECTION_MODE_PROCESS:
            result = (welt_c_fifo_population(out) <
                      welt_c_fifo_capacity(out)) &&
                     (welt_c_fifo_population(in) >= 1);
            break;
        default:result = FALSE;
            break;
    }
    return result;
}

void rndclab_img_motioncorrection::invoke() {
    switch (mode) {
        case RNDCLAB_IMG_MOTIONCORRECTION_MODE_INACTIVE: {
            mode = RNDCLAB_IMG_MOTIONCORRECTION_MODE_INACTIVE;
            break;
        }
        case RNDCLAB_IMG_MOTIONCORRECTION_MODE_PROCESS: {
            //std::cout<<"----- MOTION_EUCLIDEAN edition ----- " << std::endl;
            /* Vector from prev to cur */

            clock_t begin_c = clock();
            Mat *src_img;
            welt_c_fifo_read(in, &(src_img));
            Mat cur_grey_ori = *src_img;
            if (!cur_grey_ori.data){
                mode = RNDCLAB_IMG_MOTIONCORRECTION_MODE_INACTIVE;
                cout << "Motion Correction, there is no valid input" << endl;
                break;
            }
            cur_grey_ori = cur_grey_ori - mean(cur_grey_ori);
            Mat cur_grey;
            cvtColor(cur_grey_ori, cur_grey, 6);
            //std::cout << "cur_grey type: " << cur_grey.type() << std::endl;
            int k = frame_num;  /* Obtain the current Frame Number */
            /* Acquire the reference frame */
            if (k == 1) {
                cur_grey.copyTo(prev_grey);
            }

            Mat raw_img = cur_grey;
            //preprocessing in mc

            double min, max;
            cv::Mat filtered_image, filtered_image_gray;

            filtered_image = cur_grey;
            cv::minMaxLoc(filtered_image, &min, &max);

            filtered_image = filtered_image - mean(filtered_image)[0];
            filtered_image = ((filtered_image - min) * 255) / (max - min);

            cur_grey = filtered_image;

            // Find the width and height of the color image
            Size sz = cur_grey.size();
            int height = sz.height;
            int width = sz.width;

            double downsamp_rate = 0.6;

            Mat cur2(height, width, CV_8UC1);
            Mat curDown((int) (height * downsamp_rate),
                        (int) (width * downsamp_rate), CV_8UC1);//check
            resize(cur_grey, curDown, Size(), downsamp_rate, downsamp_rate,
                   INTER_LINEAR);
            Mat prevDown((int) (height * downsamp_rate),
                         (int) (width * downsamp_rate), CV_8UC1);//check
            resize(prev_grey, prevDown, Size(), downsamp_rate,
                   downsamp_rate, INTER_LINEAR);

            // The blue and green channels will be aligned to the red channel.
            // So copy the red channel
            //aligned_channels.push_back(channels[2].clone());


            // Define motion model
            const int warp_mode =
                    MOTION_TRANSLATION;//MOTION_AFFINE;MOTION_TRANSLATION MOTION_EUCLIDEAN

            // Set space for warp matrix.
            Mat warp_matrix;
            Mat warp_matrix2;

            // Set the warp matrix to identity.
            warp_matrix = Mat::eye(2, 3, CV_32F);

            // Set the stopping criteria for the algorithm.
            int number_of_iterations = 5000;
            double termination_eps = 0.01;//1e-10

            TermCriteria criteria(TermCriteria::COUNT + TermCriteria::EPS,
                                  number_of_iterations, termination_eps);


            // Warp the blue and green channels to the red channel

            double cc = findTransformECC_J(
                    GetGradient(prevDown),
                    GetGradient(curDown),
                    warp_matrix,
                    warp_mode,
                    criteria,
                    noArray()
            );
            //cout << k << " warp_matrix : " << warp_matrix << endl;

            warp_matrix.at<float>(0, 2) /= downsamp_rate;
            warp_matrix.at<float>(1, 2) /= downsamp_rate;

            //cout << k << " warp_matrix : " << warp_matrix << endl;
            //cout << "CC " << cc << endl;
            bool isCorrectionFailed = false;

            if (cc == -1) {
                isCorrectionFailed = true;
                printf("haha\n");
                cerr
                        << "The execution was interrupted. The correlation value is going to be minimized."
                        << endl;
                cerr
                        << "Check the warp initialization and/or the size of images."
                        << endl << flush;
            }

            // Use Affine warp when the transformation is not a Homography
            warpAffine(raw_img, cur2, warp_matrix, cur2.size(),
                       INTER_LINEAR + WARP_INVERSE_MAP);

            Mat cur2Down((int) (height * downsamp_rate),
                         (int) (width * downsamp_rate), CV_8UC1);//check
            resize(cur2, cur2Down, Size(), downsamp_rate, downsamp_rate,
                   INTER_LINEAR);

            cv::Mat corre;
            cv::matchTemplate(cur2Down, prevDown, corre, cv::TM_CCORR_NORMED);
            //cout << "trans corre: " << corre << endl;

            //cout << "Frame: " << k << "/" << 1800 << " - motion correction successfull! " << endl;
            //if (corre.at<float>(0,0) > 0.955 || k<21/*k % 5 == 0*/){
            double mean = 0;
            double summean = 0;
            /*
            //test queue
            double mean_queue =0;
            double summean_queue = 0;
            */
            double b = (double) (corre.at<float>(0, 0));
            if (k > 5) {
                //test queue
                meanQueue(queue, &mean, &summean);
                /*
                meanlist(sta,&mean,&summean);
                sta.push_back(b);
                if (sta.size() == 100)
                    sta.pop_front();
                cout << "list: " <<endl;
                showlist(sta);
                */
            }

            //test queue
            /*
            cout << "summean_queue: " << summean_queue << endl;
            cout << "mean_queue-b: " << mean_queue-b << endl;
            */
            //cout << "summean: " << summean << endl;
            //cout << "b: " << b << endl;
            //cout << "mean-b: " << mean-b << endl;

            //showlist(sta);
            bool fl = 0;
//                bool isCorrectionFailed = false;
            if ((mean - b) <= (2 * summean) && k > 20 && b >= 0.5) {
                enqueue(queue, b);
            }

            //cout << "queue: " <<endl;
            //showQueue(queue);

            // if input is simulated data, replace the next line with "if ((mean-b)<=(2*summean) || k<21){"
            // real-time if ((mean-b)<=(5*summean) || k<21){
            if ((mean - b) <= (2 * summean) || k < 5) {
                cur2.copyTo(prev_grey);
                cout << "MOTION_TRANSLATION successfull" << endl;
            } else {
                //if (corre.at<float>(0,0) < 0.955 && k>20){
                fl = 1;
                cout << "MOTION_TRANSLATION fail, try MOTION_EUCLIDEAN" << endl;
                cout << "euc corre_T: " << b << endl;
                //cout << "b: " << b << "  corre: " << corre << endl;
                const int warp_mode2 = MOTION_EUCLIDEAN;
                //Mat warp_matrix2;
                warp_matrix2 = Mat::eye(2, 3, CV_32F);
                int number_of_iterations2 = 5000;
                double termination_eps2 = 0.01;
                TermCriteria criteria2(TermCriteria::COUNT + TermCriteria::EPS,
                                       number_of_iterations2, termination_eps2);
                double cc2 = findTransformECC_J(
                        GetGradient(prevDown),
                        GetGradient(curDown),
                        warp_matrix2,
                        warp_mode2,
                        criteria2,
                        noArray());
                warp_matrix2.at<float>(0, 2) /= downsamp_rate;
                warp_matrix2.at<float>(1, 2) /= downsamp_rate;
                if (cc2 == -1) {
                    isCorrectionFailed = true;
                    cerr
                            << "The execution was interrupted. The correlation value is going to be minimized."
                            << endl;
                    cerr
                            << "Check the warp initialization and/or the size of images."
                            << endl << flush;
                }
                warpAffine(raw_img, cur2, warp_matrix2, cur2.size(),
                           INTER_LINEAR + WARP_INVERSE_MAP);
                resize(cur2, cur2Down, Size(), downsamp_rate, downsamp_rate,
                       INTER_LINEAR);
                cv::matchTemplate(cur2Down, prevDown, corre,
                                  cv::TM_CCORR_NORMED);
//                cout << "euc corre: " << corre << endl;
                //if (corre.at<float>(0,0) > 0.955){
                //    cur2.copyTo(prev_grey);
                //}
                //if (corre.at<float>(0,0) < 0.955 && k>20){
                b = (double) (corre.at<float>(0, 0));
                /*
                sta.pop_back();
                cout <<"pop_back list: "<< endl;
                showlist(sta);

                cout <<"popBack queue: "<<endl;
                //popBack(queue);
                showQueue(queue);
                */


                // if input is simulated data, replace the next line with "if ((mean-b)>(10*summean) && k>20 && sta.size() > 50){"
                // if input is real time data, replace the next line with "if ((mean-b)>(50*summean) && k>20 && sta.size() > 50){"
                if ((mean - b) > (10 * summean) && k > 5
                    && queue->size > 1) {
                    cout << "MOTION_EUCLIDEAN failed" << endl;
                    isCorrectionFailed = true;
                    prev_grey.copyTo(cur2);
                } else {
                    cout << "MOTION_EUCLIDEAN successfull" << endl;
                    /*sta.push_back(b);*/
                    cur2.copyTo(prev_grey);
                    /*
                    cout <<"push_back list: "<< endl;
                    showlist(sta);
                    */

                    enqueue(queue, b);
                    //cout <<"popBack queue: "<<endl;
                    //showQueue(queue);
                }
            }
            //cout <<"final queue: "<< endl;
            //showQueue(queue);


            //cout << "size of cur2 before: " << cur2.size() << "type: "<< cur2.type() << endl;
            //resize(cur2, cur2, Size(), 0.5, 0.5, INTER_LINEAR);
            //cout << "size of cur2 after: " << cur2.size() << "type: "<< cur2.type() << endl;
            // Mat cur2out;
            // Mat in[] = {cur2, cur2, cur2};
            // merge(in, 3, cur2out);
            //std::cout << "before output type: " << cur2out.type() << std::endl;

//            IplImage *Iimgout = NULL;
//            Iimgout = cvCreateImage(cvSize(cur2.cols, cur2.rows), 8, 1);
//            IplImage ipltemp = cur2;
//            cvCopy(&ipltemp, Iimgout);

            /* Output to preprocessing actor and signal extraction actor */
            inner_image = cur2;
            Mat *img_send = &inner_image;
            welt_c_fifo_write(out, &img_send);

//            cvReleaseImage(&src_img);
            /* Update parameters */
            //prev_match = updated_matchLoc;
            frame_num = k + 1;
            mode = RNDCLAB_IMG_MOTIONCORRECTION_MODE_PROCESS;

            /* Measure execution time */
            clock_t end_c = clock();
            double elapsed_secs = double(end_c - begin_c) / CLOCKS_PER_SEC;
            std::cout << "Motion Correction execution time :" << elapsed_secs
                      << std::endl;
//            ofstream outFile;
//            outFile.open("timeMod", ios::app);
//            outFile << elapsed_secs << endl;
//            outFile.close();

//            outFile.open("matrixMod", ios::app);
//            if (fl == 1) {
//                outFile << warp_matrix2 << endl;
//            } else
//                outFile << warp_matrix << endl;
//            outFile.close();
//
//            outFile.open("corrMod", ios::app);
//            outFile << corre.at<float>(0, 0) << endl;
//            outFile.close();
//
//            if (isCorrectionFailed == true) {
//                outFile.open("errMod", ios::app);
//                outFile << k << ": " << corre.at<float>(0, 0) << endl;
//                if (fl == 1) {
//                    outFile << warp_matrix2 << endl;
//                } else
//                    outFile << warp_matrix << endl;
//                outFile << elapsed_secs << endl;
//                outFile.close();
//            }

            break;
        }
        default:mode = RNDCLAB_IMG_MOTIONCORRECTION_MODE_INACTIVE;
            break;
    }
}

void rndclab_img_motioncorrection::reset() {
    mode = RNDCLAB_IMG_MOTIONCORRECTION_MODE_PROCESS;
}

void
rndclab_img_motioncorrection::connect(welt_cpp_graph *graph) {
//    int port_index;
//    int direction;
//    /* input 1*/
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 1;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
}

void rndclab_img_motioncorrection_terminate(
        rndclab_img_motioncorrection *context) {
    delete context;
}
