/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "rndclab_img_preprocess.h"

#define MAX_FIFO_COUNT  2

#include <iostream>

using namespace cv;
using namespace std;
extern int img_num;

int num_preprocess = 0;
Ptr<CLAHE> pFilter;
int tilesize;
int cliplimit;

rndclab_img_preprocess::rndclab_img_preprocess(
        lide_c_fifo_pointer in, lide_c_fifo_pointer out) {
    /*img input - tokens are pointers to image data*/
    this->in = (welt_c_fifo_pointer)in;
    /*img output - tokens are pointers to image data*/
    this->out = (welt_c_fifo_pointer)out;
    /* Set actor mode to ACTIVE */
    mode = RNDCLAB_IMG_PREPROCESS_MODE_ACTIVE;
    subgraph_count = 0;
    max_port_count = MAX_FIFO_COUNT;
}

rndclab_img_preprocess::~rndclab_img_preprocess() {
    cout << "delete img preprocess actor" << endl;
}

int rndclab_img_preprocess::enable() {
    boolean retval = FALSE;
    switch (mode) {
        case RNDCLAB_IMG_PREPROCESS_MODE_INACTIVE:retval = FALSE;
            break;
        case RNDCLAB_IMG_PREPROCESS_MODE_ACTIVE:
            retval = (welt_c_fifo_population(out) <
                      welt_c_fifo_capacity(out)) &&
                              (welt_c_fifo_population(in)>=1);
            break;
        default:retval = FALSE;
            break;
    }
    return retval;
}

void rndclab_img_preprocess::invoke() {
    switch (mode) {
        /* Inactive actor state */
        case RNDCLAB_IMG_PREPROCESS_MODE_INACTIVE:
            mode = RNDCLAB_IMG_PREPROCESS_MODE_INACTIVE;
            //std::cout << "inactive"<< std::endl;
            break;

            /* Active actor state */
        case RNDCLAB_IMG_PREPROCESS_MODE_ACTIVE:
            if (welt_c_fifo_population(in) == 0) {
                mode = RNDCLAB_IMG_PREPROCESS_MODE_INACTIVE;
                fprintf(stderr, "Image input unavailable\n");
                break;
            } else {
                clock_t begin = clock();
//            IplImage *src_img = nullptr;

                Mat *src_img;

                /* Get input */
                welt_c_fifo_read(in, &src_img);
                cv::Mat thresholded_image;
                cv::Mat processed_image;

                /* Convert input image to grayscale */
                Mat gray_image = *src_img;
//            cv::Mat gray_image = cv::cvarrToMat(src_img);
                double min, max;
                cv::Mat filtered_image, filtered_image_gray;

                filtered_image = gray_image;

                /* To enhance the image’s contrast, normalizing the image
                 * intensities */
                cv::minMaxLoc(gray_image, &min, &max);
                filtered_image = gray_image - mean(gray_image)[0];
                filtered_image = ((filtered_image - min) * 255) / (max - min);

                /* Apply median filter to image (Users can modify the parameter) */
                medianBlur(filtered_image, filtered_image, 5);

                /* Apply gaussian filter to image (Users can modify the parameters)
                 * */
                GaussianBlur(filtered_image, processed_image, Size(3, 3), 1);

                /* Write the destination image (Mat* format) to out */
                inner_image = processed_image;
                Mat *img_send = &inner_image;
                welt_c_fifo_write(out,&img_send);

                clock_t end = clock();
                double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
                std::cout << "Preprocessing execution time :" << elapsed_secs
                          << std::endl;

                mode = RNDCLAB_IMG_PREPROCESS_MODE_ACTIVE;

            }
            break;

        default:mode = RNDCLAB_IMG_PREPROCESS_MODE_INACTIVE;
            break;
    }
}

void rndclab_img_preprocess::reset() {
    mode = RNDCLAB_IMG_PREPROCESS_MODE_ACTIVE;
}

void rndclab_img_preprocess::connect(welt_cpp_graph *graph) {
//    int port_index;
//    int direction;
//    /* input 1*/
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 1;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
}

void rndclab_img_preprocess_terminate(
        rndclab_img_preprocess *context) {
    delete context;
}
