#ifndef _rndclab_img_preprocess_h
#define _rndclab_img_preprocess_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

extern "C" {
#include "lide_c_util.h"
#include "lide_c_actor.h"
#include "lide_c_fifo.h"
//#include "lide_c_fifo_basic.h"
#include "welt_c_fifo.h"
}

#include "welt_cpp_actor.h"
#include "welt_cpp_graph.h"

#include <opencv2/core.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc.hpp>

#define RNDCLAB_IMG_PREPROCESS_MODE_INACTIVE              1
#define RNDCLAB_IMG_PREPROCESS_MODE_ACTIVE                2
#define RNDCLAB_IMG_PREPROCESS_MODE_TRANSITION_DETECTION  3

using namespace cv;

class rndclab_img_preprocess : public welt_cpp_actor{
public:
    rndclab_img_preprocess(
            lide_c_fifo_pointer in, lide_c_fifo_pointer out);

    virtual ~rndclab_img_preprocess();
    /* Enable function */
    int enable() override;
    /* Invoke function */
    void invoke() override;

    void reset() override;

    void connect(welt_cpp_graph *graph) override;

private:
    /*Actor interface ports.*/
    welt_c_fifo_pointer in;
    welt_c_fifo_pointer out;

    /* Inner state variable prevent from being auto released */
    Mat inner_image;
};

/* Terminate function */
void rndclab_img_preprocess_terminate(
        rndclab_img_preprocess *context);

#endif

