#ifndef _rndclab_img_read_h
#define _rndclab_img_read_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

extern "C" {
#include "lide_c_util.h"
#include "lide_c_actor.h"
#include "lide_c_fifo.h"
//#include "lide_c_fifo_basic.h"
#include "welt_c_fifo.h"
}
#include "welt_cpp_actor.h"
#include "welt_cpp_graph.h"

#include <opencv2/core.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/highgui.hpp>

/* define the inactive mode for source actor */
#define RNDCLAB_IMG_READ_MODE_INACTIVE	1
/* read the image from the specified directory and update the image source
 * name*/
#define RNDCLAB_IMG_READ_MODE_SRC_UPDATE	2

class rndclab_img_read : public welt_cpp_actor{
public:
    rndclab_img_read(char *first_file,
                             int img_num,
                             lide_c_fifo_pointer img_out);

    virtual ~rndclab_img_read();

    int enable() override;

    void invoke() override;

    void reset() override;

    void connect(welt_cpp_graph *graph) override;

private:
    /* the file name of the first image in the video stream */
    char *first_file;
    /* Actor interface ports. */
    welt_c_fifo_pointer out;
    /* total number of image */
    int img_num;
    char file_name_val[80];
    /* temp variable to hold the current file name */
    char *temp;
    /* index of the image under processing */
    int img_index;
    cv::Mat img_in;
};

void rndclab_img_read_terminate(
        rndclab_img_read *context);

#endif
