/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
/*This actor takes the number of detected neurons from the BODY graph and
 * reconfigure the parameters in initialization mode. In real-time mode, this
 * actor will be disabled.*/
#include <iostream>

#include "rndclab_set_param.h"

#define MAX_FIFO_COUNT  3
#define PARAM_COUNT_SUB 4
/* First 200 frames are using for the initialization mode*/
#define NUMFRAMEFORINITIAL 200

using namespace std;

rndclab_set_param::rndclab_set_param
(welt_cpp_actor *target, int detectedResult,int target_param_index) {
    this->target = target;
    this->frame_num_setParam = 0;
    /* old way passing parameters */
//    this->target_param_index = target_param_index;
//    this->target_param_length = target->param_length[target_param_index];
//    this->inter_fifo = lide_c_fifo_basic_new(2048, sizeof(int), 0);
    this->mode = LIDE_C_SET_PARAM_MODE_PROCESS;
    this->param = new int[4];
    /* C style initialize */
//    this->param = (int *) lide_c_util_malloc(sizeof(int) * (4));

}

rndclab_set_param::~rndclab_set_param() {
    cout << "delete set param actor" << endl;
}

int rndclab_set_param::enable() {
    boolean result = FALSE;
    switch (mode) {
        case LIDE_C_SET_PARAM_MODE_INACTIVE:
            result = FALSE;
            break;
        case LIDE_C_SET_PARAM_MODE_PROCESS:result = TRUE;
            break;
        default:result = FALSE;
            break;
    }
    return result;
}

void rndclab_set_param::invoke() {
    int i;
    switch (mode) {
        case LIDE_C_SET_PARAM_MODE_INACTIVE: {

            mode = LIDE_C_SET_PARAM_MODE_INACTIVE;
            break;
        }
        case LIDE_C_SET_PARAM_MODE_PROCESS: {

            if (frame_num_setParam < NUMFRAMEFORINITIAL) {

                param[0] = 0;
            } else {
                if (frame_num_setParam == NUMFRAMEFORINITIAL) {
                    std::cout << "----------Actor SetParam RealTime Mode"
                                 "----------"
                              << std::endl;
                }
                param[0] = 1;
                mode = LIDE_C_SET_PARAM_MODE_INACTIVE;
                std::cout << "----------Actor SetParam disabled----------"
                << std::endl;
            }
            bool changeInfo = false;
            /*Initialize the parameters*/
            if (frame_num_setParam == 0) {
                std::cout << "----------Actor SetParam Initialization Mode"
                             "----------"
                          << std::endl;
                param[0] = 0;
                param[1] = 30;  /* threshold step */
                param[2] = 30;   /* Min area */
                param[3] = 300;   /* Max area */
            }
            std::cout << "params in set param actor: " << param[0] << " "
                      << param[1] << " " << param[2] << " "
                      << param[3] << " " << std::endl;
            /* set parameters */
            /* Users can modify the numbers. Current version is using the
             * following method to reconfiguration; Starting from the
             * initialized parameters, in every 20 frames, if the number of
             * detected neurons are less than 5, it will reduce the threshold
             * step by 20, reduce the minimum area by 5 and increase the max
             * area by 20 to get more neurons. This way, the criteria for the
             * detection will be loosened. */

            /* get number of detected neurons */
            auto num_detected = (int*)(target->params_vec[4]);

            if (frame_num_setParam % 20 == 1) {
                if (param[0] == 0) {
                    if (*num_detected < 5) {
                        if (param[1] > 20) {
                            param[1] -= 20;
                            changeInfo = true;
                        }
                        if (param[1] < 400) {
                            param[2] -= 5;
                            changeInfo = true;
                        }
                        if (param[2] > param[1]) {
                            param[3] += 20;
                            changeInfo = true;
                        }
                    }
                }
            }
            if (changeInfo) {
                std::cout << "Parameter changed: " << param
                          << std::endl;
                //last_keypoints.clear();
                changeInfo = false;
                std::cout << "new params in set param actor: " << param[0] <<
                " "
                          << param[1] << " " << param[2] << " "
                          << param[3] << " " << std::endl;
            }

            /* new way passing parameters */
            for (int idx = 0; idx < 4; idx++) {
                auto tmp = (int*)(target->params_vec[idx]);
                *tmp = param[idx];
            }

            /* old way passing parameters */
//            lide_c_fifo_basic_write(inter_fifo, &(param[0]));
//            lide_c_fifo_basic_write(inter_fifo, &(param[1]));
//            lide_c_fifo_basic_write(inter_fifo, &(param[2]));
//            lide_c_fifo_basic_write(inter_fifo, &(param[3]));
//
//            for (i = 0; i < target_param_length; i++) {
//                lide_c_fifo_basic_read(
//                        inter_fifo,
//                        (int *) (target->paramrefs[target_param_index]) + i);
//            }

            frame_num_setParam++;
            mode = LIDE_C_SET_PARAM_MODE_PROCESS;
            break;
        }
        default:mode = LIDE_C_SET_PARAM_MODE_INACTIVE;
            break;
    }
}

void rndclab_set_param::reset() {
    mode = LIDE_C_SET_PARAM_MODE_PROCESS;
}

void rndclab_set_param::connect(welt_cpp_graph *graph) {
//    int port_index;
//    int direction;
//
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *)context,
//                                port_index, direction);
//
//
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 1;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *)context,
//                                port_index, direction);
//
//
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 2;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *)context,
//                                port_index, direction);
}

void
rndclab_set_param_terminate(rndclab_set_param *context){
    delete context;
}

