#ifndef _rndclab_set_param_h
#define _rndclab_set_param_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

extern "C" {
#include "lide_c_util.h"
#include "lide_c_actor.h"
#include "lide_c_fifo.h"
//#include "lide_c_fifo_basic.h"
#include "welt_c_fifo.h"
}

#include "welt_cpp_actor.h"
#include "welt_cpp_graph.h"


/* Actor modes */
#define LIDE_C_SET_PARAM_MODE_PROCESS        1
#define LIDE_C_SET_PARAM_MODE_INACTIVE     2

/*******************************************************************************
 TYPE DEFINITIONS
 *******************************************************************************/

class rndclab_set_param : public welt_cpp_actor{
public:
/*******************************************************************************
 INTERFACE FUNCTIONS
 *******************************************************************************/

/*****************************************************************************
 Construct function of the lide_c_set_param actor. Create a new
 lide_c_set_param with the specified input FIFO pointer, and the specified
 actor pointer. Data format in the input FIFOs should be integers.
 *****************************************************************************/
    rndclab_set_param(welt_cpp_actor *target, int
    detectedResult, int target_param_index);

    virtual ~rndclab_set_param();

/*****************************************************************************
 Enable function of the lide_c_set_param actor.
 *****************************************************************************/
    int enable() override;
/*****************************************************************************
 Invoke function of the lide_c_set_param actor.
 *****************************************************************************/
    void invoke() override;

    void reset() override;

    void connect(welt_cpp_graph *graph) override;

private:
    welt_cpp_actor *target;
    int frame_num_setParam;
    /* old way passing parameters */
//    int target_param_index;
//    int target_param_length;
    /*Actor interface ports.*/
//    lide_c_fifo_basic_pointer inter_fifo;

    /* an array to key parameters. param[0]-> indicates the current mode. 0 for
     * initialization mode and 1 for real-time mode */
    int *param;
};

/*****************************************************************************
 Terminate function of the lide_c_set_param actor.
 *****************************************************************************/
void rndclab_set_param_terminate(rndclab_set_param *context);

#endif
