/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 2017-2020
UMB-UMD Neuromodulation Research Group,
University of Maryland at Baltimore, and 
University of Maryland at College Park. 

All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BALTIMORE
OR UNIVERSITY OF MARYLAND COLLEGE PARK BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "rndclab_signal_extraction.h"

#define MAX_FIFO_COUNT  3

#include <opencv2/imgproc.hpp>
#include <opencv2/core.hpp>

using namespace cv;
using namespace std;

rndclab_signal_extraction::rndclab_signal_extraction(
        lide_c_fifo_pointer img_in, lide_c_fifo_pointer sig_in,
        lide_c_fifo_pointer signal_out) {
    this->mode = LIDE_C_SIGNAL_EXTRACTION_MODE_PROCESS;
    this->img_num = 0;

    /*img input - tokens are pointers to image data*/
    this->img_in = (welt_c_fifo_pointer) img_in;
    this->sig_in = (welt_c_fifo_pointer) sig_in;
    this->signal_out = (welt_c_fifo_pointer) signal_out;

    this->subgraph_count = 0;
    this->max_port_count = MAX_FIFO_COUNT;
}

rndclab_signal_extraction::~rndclab_signal_extraction() {
    cout << "delete signal extraction actor" << endl;
}

int rndclab_signal_extraction::enable() {
    boolean result = FALSE;
    switch (mode) {
        case LIDE_C_SIGNAL_EXTRACTION_MODE_INACTIVE:result = FALSE;
            break;
        case LIDE_C_SIGNAL_EXTRACTION_MODE_PROCESS:
            result = (welt_c_fifo_population(signal_out) <
                      welt_c_fifo_capacity(signal_out)) &&
                     (welt_c_fifo_population(img_in)>=1) &&
                     (welt_c_fifo_population(sig_in)>=1);
            break;
        default:result = FALSE;
            break;
    }
    return result;
}

void rndclab_signal_extraction::invoke() {
    switch (mode) {
        case LIDE_C_SIGNAL_EXTRACTION_MODE_INACTIVE:
            mode = LIDE_C_SIGNAL_EXTRACTION_MODE_INACTIVE;
            break;
        case LIDE_C_SIGNAL_EXTRACTION_MODE_PROCESS:
            if (welt_c_fifo_population(img_in) == 0) {
                mode = LIDE_C_SIGNAL_EXTRACTION_MODE_INACTIVE;
                break;
            } else {
                clock_t begin = clock();

                Mat mask;
                Mat value_mask;

                /* Get input from input fifo */
                Mat *Inewimg;

                /* Get input */
                welt_c_fifo_read(img_in, &Inewimg);

                std::vector<KeyPoint>
                        *last_keypoints_se = new std::vector<KeyPoint>();
                welt_c_fifo_read(sig_in,&last_keypoints_se);
                cv::Mat newimg = *Inewimg;

                /* Compute signal f-f0/f0 */
                /* Depending on the characteritics of their own data, user can
                 * define f0. In the current code, we defined the average value of
                 * intensities of whole pixels in the frame. Also, users can use the
                 * below code for f0 as "average ROI intensities across a window of
                 * time that immediately precedes a particular experimental event."
                 * */
                /* vector<double> neuron_temp;

                 neuron_temp.assign(neuron[k].begin(),
                        neuron[k].begin()+NumberOfFrameInWindow);

                 std::sort (neuron_temp.begin(), neuron_temp.end());

                 f0= neuron_temp[10%ofNumberOfFrameInWindow]; */

                long sum;
                long f_0_long = 0;
                double f_0;
                double signal_double;
                double neuron_online[900] = {};
                int t;
                double value;
                int n, r, c, k;

                for (k = 0; k < last_keypoints_se->size(); k++) {

                    value = 0;
                    sum = 0;
                    n = 0;

                    circle(mask, Point((*last_keypoints_se)[k].pt.x,
                                       (*last_keypoints_se)[k].pt.y),
                           (*last_keypoints_se)[k].size / 2, Scalar(255), -1);

                    newimg.copyTo(value_mask, mask);
                    value = mean(value_mask)[0];

                    /* Usually it means the average value of previous signals or if
                     * for removing trend, the average value of intensities of whole
                     * pixel in the frame */

                    f_0 = mean(mask)[0];
                    signal_double = (value - f_0) / f_0;
                    mask = Mat::zeros(newimg.rows, newimg.cols, CV_8UC1);
                    value_mask = Mat::zeros(newimg.rows, newimg.cols, CV_8UC1);

                    /* Offline */
                    //neuron[k][num_extraction]= signal_double;

                    /* Online */
                    neuron_online[k] = signal_double;

                } /* End of for loop */


                /* Write the output signal using fifo_write_block*/
                /* Considering the number of detected neuron is at most 200 */
                FILE *fp = fopen("signals.txt", "a");
                img_num++;
                fprintf(fp,"frame_%d: ",img_num);
                for (int idx = 1; idx<last_keypoints_se->size(); idx++) {
                    fprintf(fp,"%f ",neuron_online[idx]);
                }
                fprintf(fp,"\n");
                fclose(fp);
//            ofstream outFile;
//            outFile.open("signals.txt", ios::app);
//            for (int idx = 1; idx<last_keypoints_se->size(); idx++) {
//                outFile << neuron_online[idx] << " ";
//            }
//            outFile << endl;
//            outFile.close();

                welt_c_fifo_write_block(signal_out,&neuron_online, 200);

                //num_extraction++;
                clock_t end = clock();
                double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
                std::cout << "Signal extraction execution time :" << elapsed_secs
                          << std::endl;
                mode = LIDE_C_SIGNAL_EXTRACTION_MODE_PROCESS;
                k++;


            }
            break;

        default:mode = LIDE_C_SIGNAL_EXTRACTION_MODE_INACTIVE;
            break;
    }
}

void rndclab_signal_extraction::reset() {
    mode = LIDE_C_SIGNAL_EXTRACTION_MODE_PROCESS;
}

void rndclab_signal_extraction::connect(welt_cpp_graph *graph) {
//    int port_index;
//    int direction;
//
//    /* input 1*/
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//
//    /* input 2*/
//    direction = GRAPH_IN_CONN_DIRECTION;
//    port_index = 1;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
//
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 2;
//    lide_c_graph_add_connection(graph, (lide_c_actor_type *) context,
//                                port_index, direction);
}

void rndclab_signal_extraction_terminate(
        rndclab_signal_extraction *context) {
    delete context;
}
