/***************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2018
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>

/*******************************************************************************
dx (dice) in (insert) cr (carriage returns) core
This program derives an output file by replacing every instance of the
character sequence "\r" with a carriage return character, and copying all other
characters without any change.  The input file is left unchanged.  A status
message indicating the number of "\r" sequences replaced is printed to standard
output.

Usage: dxincrcore.exe <input_file> <output_file>
*******************************************************************************/

/* Suppress "core" for conciseness */
#define PROG "dxincr"

int main(int argc, char **argv) {
    const char s1 = '\\'; /* source character 1 */ 
    const char s2 = 'r'; /* source character 2 */
    const char r = '\r';  /* replacement character */
    char c = '\0';
    int count = 0;
    FILE *input = NULL;
    FILE *output = NULL;
        
    if (argc != 3) {
        fprintf(stderr, "%s error: arg count", PROG);
        exit(1);
    }
    input = fopen(argv[1], "r");
    output = fopen(argv[2], "w");
    if (input == NULL) {
        fprintf(stderr, "%s error: Could not open input file '%s'.\n", 
                PROG, argv[1]);
        exit(1);
    }
    if (output == NULL) {
        fprintf(stderr, "%s error: Could not open output file '%s'.\n", 
                PROG, argv[1]);
        exit(1);
    }

    while ((c = fgetc(input)) != EOF) {
        if (c == s1) {
            c = fgetc(input);
            if (c == s2) {
                fputc(r, output);
                count++;
            } else if (c == EOF) {
                fputc(s1, output);
            } else {
                fputc(s1, output);
                fputc(c, output);
            }
        } else {
            fputc(c, output);
        }
    }

    fclose(input);
    fclose(output);

    printf("%s: %d \\r string(s) replaced in '%s'.\n", 
            PROG, count, argv[1]);

    return 0;
}

