#ifndef _lide_c_cluster_isolation_h
#define _lide_c_cluster_isolation_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2018
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include "lide_c_actor.h"
#include "lide_c_fifo.h"

/*******************************************************************************
This actor reads grid data and compute the tau and electron isolations.
*******************************************************************************/

/* Macro definition */
#define COUNT_THRESHOLD(CoeffA, CoeffB, CompressedEnergy)  (int) (CoeffA + (double) (CoeffB * (CompressedEnergy * 16 + 8)) / 1000)

/* Actor modes */
#define LIDE_C_CLUSTER_ISOLATION_MODE_ERROR            0
#define LIDE_C_CLUSTER_ISOLATION_MODE_PROCESS          1

/* Constants */
#define GRID_LENGTH     8
#define GRID_HEIGHT     8
#define GRID_SIZE       64

/*******************************************************************************
TYPE DEFINITIONS
*******************************************************************************/

/* Structure and pointer types associated with CLUSTER ISOLATION objects. */
struct _lide_c_cluster_isolation_context_struct;
typedef struct _lide_c_cluster_isolation_context_struct
        lide_c_cluster_isolation_context_type;

/*******************************************************************************
INTERFACE FUNCTIONS
*******************************************************************************/

/*****************************************************************************
Construct function of the lide_c_cluster_isolation actor.
*****************************************************************************/
lide_c_cluster_isolation_context_type *lide_c_cluster_isolation_new(
        lide_c_fifo_pointer ffp_in_grid, 
        lide_c_fifo_pointer ffp_out_tau_isolation, 
        lide_c_fifo_pointer ffp_out_electron_isolation);

/*****************************************************************************
Enable function of the lide_c_cluster_isolation actor.
*****************************************************************************/
boolean lide_c_cluster_isolation_enable(
        lide_c_cluster_isolation_context_type *context);

/*****************************************************************************
Invoke function of the lide_c_cluster_isolation actor.
*****************************************************************************/
void lide_c_cluster_isolation_invoke(
        lide_c_cluster_isolation_context_type *context);

/*****************************************************************************
Terminate function of the lide_c_cluster_isolation actor.
*****************************************************************************/
void lide_c_cluster_isolation_terminate(
        lide_c_cluster_isolation_context_type *context);

#endif
