/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <iostream>

#include "welt_cpp_file_source.h"

#define MAX_FIFO_COUNT  1

using namespace std;

// constructor
welt_cpp_file_source::welt_cpp_file_source
        (lide_c_fifo_pointer out, char *file, int index,
         int token_type) {
    this->token_type = token_type;
    this->file = file;
    this->fp = lide_c_util_fopen((const char *)this->file, "r");
    switch (this->token_type) {
        case WELT_CPP_TOKEN_TYPE_PRIMITIVE:
            if (fscanf(this->fp, "%d", &this->data) != 1) {
                /* End of input */
                this->mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
                this->data = 0;
            }else{
                this->mode = WELT_CPP_FILE_SOURCE_MODE_WRITE;
            }
            break;
        case WELT_CPP_TOKEN_TYPE_POINTER: {
            /* read all integers from input file, and store them in a buffer */
            int tmp_read_data;
            /* try to read the first integer */
            if (fscanf(this->fp, "%d", &tmp_read_data) != 1) {
                /* End of input */
                this->mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
            }else{
                this->mode = WELT_CPP_FILE_SOURCE_MODE_WRITE;
                /* push the first integer to the buffer */
                this->data_buffer.push_back(tmp_read_data);
                /* read and push the rest of integers */
                while (fscanf(this->fp, "%d", &tmp_read_data) == 1) {
                    this->data_buffer.push_back(tmp_read_data);
                }
                /* initialize the iterator pointing to the first element */
                this->iter = this->data_buffer.begin();
            }
            break;
        }
        case WELT_CPP_TOKEN_TYPE_OBJECT:
            break;
        default:
            break;
    }


    this->out = (lide_c_fifo_basic_pointer)out;

    this->index = index;
//    this->subgraphs = nullptr;
    this->subgraph_count = 0;
    this->max_port_count = MAX_FIFO_COUNT;

    //    FIXME: port init_portrefs and add_portrefs functions to cpp
    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context,
        FIFO_COUNT, &context->in);
    */
//    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
//    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context,
//                              (lide_c_fifo_pointer *)&context->in);
}

boolean welt_cpp_file_source::enable() {
    boolean result = FALSE;

    switch (mode) {
        case WELT_CPP_FILE_SOURCE_MODE_WRITE:
            result = (lide_c_fifo_basic_population(out) <
                      lide_c_fifo_basic_capacity(out));
            break;
        case WELT_CPP_FILE_SOURCE_MODE_INACTIVE:
            result = FALSE;
            break;
        default:
            result = FALSE;
            break;
    }
    return result;
}

void welt_cpp_file_source::invoke() {
    switch (mode) {
        case WELT_CPP_FILE_SOURCE_MODE_WRITE:
//            lide_c_fifo_basic_write(
//                    (lide_c_fifo_basic_pointer)*(context->portrefs[0]),
//                    &context->data);
            switch (token_type) {
                case WELT_CPP_TOKEN_TYPE_PRIMITIVE:
                    lide_c_fifo_basic_write(out, &data);
                    if (fscanf(fp, "%d", &data) != 1) {
                        /* End of input */
                        mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
                    }else{
                        mode = WELT_CPP_FILE_SOURCE_MODE_WRITE;
                    }
                    break;
                case WELT_CPP_TOKEN_TYPE_POINTER: {
                    /* If the iterator hits the end, there is no more data to
                     * be send to fifo. The actor will be inactivated. */
                    if (iter == data_buffer.end()){
                        /* End of input */
                        mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
                        break;
                    }
                    /* write the iterator (which is a pointer to the element
                     * to be output) to the fifo */
                    lide_c_fifo_basic_write(out, &iter);
                    /* iterator moves forward to the next element */
                    iter++;
                    break;
                }
                case WELT_CPP_TOKEN_TYPE_OBJECT:
//                    FIXME
                    break;
                default:
                    printf("the token_type is not set properly in #%d actor\n",
                           index);
                    mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
                    break;
            }

            break;
        case WELT_CPP_FILE_SOURCE_MODE_INACTIVE:
            mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
            break;
        default:
            mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
            break;
    }

//    lide_c_fifo_basic_write(out, &img_send);
}

void welt_cpp_file_source::reset() {
    fclose(fp);
    fp = lide_c_util_fopen((const char *)file, "r");
    if (fscanf(fp, "%d", &data) != 1) {
        mode = WELT_CPP_FILE_SOURCE_MODE_INACTIVE;
        data = 0;

    }else{
        mode = WELT_CPP_FILE_SOURCE_MODE_WRITE;
    }
}

void welt_cpp_file_source::connect(welt_cpp_graph *graph) {
    int port_index;
    int direction;

//    following lines are waiting for the util functions being ported
//    /* output 1*/
//    direction = GRAPH_OUT_CONN_DIRECTION;
//    port_index = 0;
//    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context,
//                                port_index, direction);
}

welt_cpp_file_source::~welt_cpp_file_source() {
    switch (token_type) {
        case WELT_CPP_TOKEN_TYPE_PRIMITIVE:
            fclose(fp);
            break;
        default:
            break;
    }
    cout << "delete file source actor" << endl;
}

void welt_cpp_file_source_terminate(welt_cpp_file_source
*context) {
    delete context;
}
