#ifndef _welt_cpp_graph_h
#define _welt_cpp_graph_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <stdio.h>
#include <queue>
#include "lide_c_fifo.h"

#define GRAPH_IN_CONN_DIRECTION  0   /* input port of an actor*/
#define GRAPH_OUT_CONN_DIRECTION 1   /* output port of an actor*/

using namespace std;

/* This is called a "forward declaration".  We use it to tell the compiler that
 * the identifier "welt_cpp_actor_context.h" will from now on stand for a class,
 * and this class will be defined later.  We will not be able to make any use of
 * "welt_cpp_actor_context.h" before it has been defined, but we will at least
 * be able to declare pointers to it. */
class welt_cpp_actor;

class welt_cpp_graph{
public:
/* Common field elements in the graph. */
    /* A collection of fifos */
    vector<lide_c_fifo_pointer> fifos;
    /* A collection of actor pointers */
    vector<welt_cpp_actor*> actors;
    /* A collection of descriptors */
    vector<char*> descriptors;
    /* The number of actors */
    int actor_count;
    /* The number of fifos */
    int fifo_count;

//    FIXME: source array and sink array
    /* following two members are not tested */
    vector<welt_cpp_actor*> source_array;
    vector<welt_cpp_actor*> sink_array;

    /* The scheduler method of a welt-cpp graph corresponds to the scheduler of
     * the lightweight/welterweight dataflow API. Every actor must have a
     * scheduler function that executes the dataflow graph */
    virtual void scheduler() = 0;

    void welt_cpp_graph_add_actor();
    void welt_cpp_graph_add_connection(
            welt_cpp_actor *context, int port_index,
            int direction);
    void lide_c_graph_update_connection(
            welt_cpp_actor *context, int port_index,
            int fifo_index, int direction);
    void welt_cpp_graph_set_actor(int index, welt_cpp_actor *actor_context);
    welt_cpp_actor *welt_cpp_graph_get_actor(int index);
    void welt_cpp_graph_set_fifo(int index, lide_c_fifo_pointer fifo);
    lide_c_fifo_pointer welt_cpp_graph_get_fifo(int index);
    void welt_cpp_graph_set_actor_count(int count);
    int welt_cpp_graph_actor_count();
    int welt_cpp_graph_fifo_count();
    vector<welt_cpp_actor*>* welt_cpp_graph_get_sink_array();
    vector<welt_cpp_actor*>* welt_cpp_graph_get_source_array();
    void welt_cpp_graph_graph_add_fifo();
    };
#endif // _welt_cpp_graph_h
