/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "lide_c_add_graph.h"
#include "lide_c_dsg.h"
#include "lide_c_util.h"
#include "lide_c_add_shched2.h"
#include "lide_c_fifo.h"
//#include "lide_c_cdsg_util.h"

pthread_mutex_t *mutex;
pthread_cond_t *cond;

#define NUM_THREAD 2


typedef struct {
    int id;
    lide_c_graph_context_type **app_graph;
    lide_c_dsg_context_type **dsg_graph;
}parm;

void *pthread_scheduler(void *arg);
/* 
    Usage: lide_c_add_driver.exe x_file y_file out_file
*/

int main(int argc, char **argv) {
    
    
    mutex = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
    cond = (pthread_cond_t*)malloc(sizeof(pthread_cond_t));
    
    if (pthread_mutex_init(mutex, NULL) != 0)
    {
        printf("\n mutex init failed\n");
        return 1;
    }
    
    if (pthread_cond_init(cond, NULL) != 0)
    {
        printf("\n mutex_cond init failed\n");
        return 1;
    }
    
    lide_c_graph_context_type *app_graph = NULL;
    lide_c_graph_context_type *dsg_graph = NULL;
    lide_c_dsg_context_type *dsg = NULL;
    
    char *x_file = NULL; 
    char *y_file = NULL; 
    char *out_file = NULL; 
    
    int i = 0;
    int arg_count = 4;

    //lide_c_fifo_pointer new_fifo;
    
    /* Posix thread related variable(s). */
    pthread_t *threads;
    pthread_attr_t pthread_custom_attr;
    parm *p;
    
    /* Posix thread configuration(s). */
    
    threads=(pthread_t *)malloc(NUM_THREAD * sizeof(*threads));
    pthread_attr_init(&pthread_custom_attr);
    
    p=(parm *)malloc(sizeof(parm)*NUM_THREAD);
    
    
    /* Check program usage. */
    if (argc != arg_count) {
        fprintf(stderr, "lide_c_add_driver.exe error: arg count\n");
        exit(1);
    }   

    /* Open the input and output file(s). */
    i = 1;
    x_file = argv[i++]; 
    y_file = argv[i++]; 
    out_file = argv[i++];
    
    //printf("graph \n");
    app_graph = (lide_c_graph_context_type *)
                    lide_c_add_graph_new(x_file, y_file, out_file);
    //printf("dsg_graph \n");
    dsg = (lide_c_dsg_context_type *) lide_c_add_shched2_new(app_graph,mutex,cond);
    
    for (i=0; i<NUM_THREAD; i++)
    {
        p[i].id=i;
        p[i].app_graph = &app_graph;
        p[i].dsg_graph = &dsg;
    }
    
    for (i=0; i<NUM_THREAD; i++)
    {
        pthread_create(&threads[i], &pthread_custom_attr, pthread_scheduler,
                       (void *)(p+i));
    }
    
    for (i=0; i<NUM_THREAD; i++)
    {
        pthread_join(threads[i],NULL);
    }
    free(p);
   
    return 0;
}

void *pthread_scheduler(void *arg)
{
    parm *p=(parm *)arg;
    
    if (p->id == 0){
        /* Execute graph*/
        //printf("id 0\n");
    lide_c_add_shched2_scheduler((lide_c_add_shched2_context_type*)*(p->dsg_graph),0,NUM_THREAD);
    }
    else if (p->id == 1){
        /* Execute graph*/
        //printf("id 1\n");
    lide_c_add_shched2_scheduler((lide_c_add_shched2_context_type*)*(p->dsg_graph),1,NUM_THREAD);

    }
    
    /* Execute graph */
    return (NULL);
}
