/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <sys/types.h>

#include "lide_c_add_shched2.h"
#include "lide_c_dsg_scheduler.h"

#include "lide_c_basic.h"
#include "lide_c_actor.h"
#include "lide_c_add_graph.h"
#include "lide_c_cdsg_util.h"


struct _lide_c_add_shched2_context_struct {
    #include "lide_c_dsg_context_type_common.h"
    
};

lide_c_add_shched2_context_type *lide_c_add_shched2_new(
        lide_c_graph_context_type * contextApp, pthread_mutex_t *mutex,
        pthread_cond_t *cond){
    int i;
    
    lide_c_add_shched2_context_type * add_shched2 = NULL;
   
    add_shched2 = (lide_c_add_shched2_context_type *) lide_c_dsg_new(DSG_ACTOR_COUNT,2);

    lide_c_graph_context_type * context = NULL;
    context = (lide_c_graph_context_type*)lide_c_util_malloc(
                sizeof(lide_c_graph_context_type));
    
    /* Memory allocation foradd_shched2 */
    context->fifo_count = DSG_FIFO_COUNT;
    context->actor_count = DSG_ACTOR_COUNT;
    
    context->actors = (lide_c_actor_context_type **)lide_c_util_malloc
            (context->actor_count * sizeof(lide_c_actor_context_type *));
    context->fifos = (lide_c_fifo_pointer *)lide_c_util_malloc
            (context->fifo_count * sizeof(lide_c_fifo_pointer));
    context->descriptors = (char **)lide_c_util_malloc(
           context->actor_count * sizeof(char*));
    
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_LAE1, "lae1");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_XREF, "x_ref");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_YREF, "y_ref");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_SND, "snd");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_LAE2, "lae2");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_REC, "rec");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_ADDSCA, "sca_add");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_ADDREF, "ref_add");
    lide_c_graph_set_actor_desc((lide_c_graph_context_type *)context,
                                DSG_ACTOR_SNKREF, "ref_snk");
    
    context->source_array = (lide_c_actor_context_type **)
            lide_c_util_malloc(context->fifo_count *
            sizeof(lide_c_actor_context_type *));
    
    context->sink_array = (lide_c_actor_context_type **)
            lide_c_util_malloc(context->fifo_count *
            sizeof(lide_c_actor_context_type *));
    
    int token_size = sizeof(int);
     for (i =0; i<DSG_FIFO_COUNT; i++){
            context->fifos[i] = (lide_c_fifo_pointer)
                lide_c_fifo_unit_size_new(token_size, i);
    }
    
    /* DSG actor LAE1 */
    context->actors[DSG_ACTOR_LAE1] = (lide_c_actor_context_type *)(lide_c_sca_sch_loop_new(
            context->fifos[DSG_FIFO_SND_LAE1] ,
            context->fifos[DSG_FIFO_LAE1_XREF], 1, DSG_ACTOR_LAE1));
    lide_c_dsg_add_actor(SCA_SCH_LOOP,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_LAE1], DSG_ACTOR_LAE1);
    lide_c_dsg_set_first_actor((lide_c_dsg_context_type*)add_shched2, context->actors[DSG_ACTOR_LAE1],DSG_ACTOR_LAE1);

    /* DSG actor Xsource ref */
    context->actors[DSG_ACTOR_XREF] = (lide_c_actor_context_type *)(lide_c_ref_actor_new(
           context->fifos[DSG_FIFO_LAE1_XREF] ,
           context->fifos[DSG_FIFO_XREF_YREF], DSG_ACTOR_XREF,
        contextApp->actors[ACTOR_XSOURCE], 0, 0, NULL, NULL, NULL ));
    lide_c_dsg_add_actor(REF,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_XREF], DSG_ACTOR_XREF);

    /* DSG actor Ysource ref */
    context->actors[DSG_ACTOR_YREF] = (lide_c_actor_context_type *)(lide_c_ref_actor_new(
           context->fifos[DSG_FIFO_XREF_YREF],
           context->fifos[DSG_FIFO_YREF_SND], DSG_ACTOR_YREF,
            contextApp->actors[ACTOR_YSOURCE], 0, 0, NULL, NULL, NULL ));
    lide_c_dsg_add_actor(REF, context, context->actors[DSG_ACTOR_YREF], DSG_ACTOR_YREF);
    
    /* DSG actor SND */
    context->actors[DSG_ACTOR_SND] = (lide_c_actor_context_type *)(lide_c_snd_actor_new(
            context->fifos[DSG_FIFO_YREF_SND],
            context->fifos[DSG_FIFO_SND_LAE1],
            context->fifos[DSG_FIFO_SND_REC] ,
            mutex, cond,
        contextApp->fifos[FIFO_YSRC_ADD], 1, DSG_ACTOR_SND));
    lide_c_dsg_add_actor(SND,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_SND], DSG_ACTOR_SND);

    /* DSG actor LAE2 */

    context->actors[DSG_ACTOR_LAE2] = (lide_c_actor_context_type *)(lide_c_sca_sch_loop_new(
            context->fifos[DSG_FIFO_SNKREF_LAE2] ,
            context->fifos[DSG_FIFO_LAE2_REC], 1, DSG_ACTOR_LAE2));
    lide_c_dsg_add_actor(SCA_SCH_LOOP,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_LAE2], DSG_ACTOR_LAE2);
    lide_c_dsg_set_first_actor((lide_c_dsg_context_type*)add_shched2, context->actors[DSG_ACTOR_LAE2],DSG_ACTOR_LAE2);

    /* DSG actor REC */
    context->actors[DSG_ACTOR_REC] = (lide_c_actor_context_type *) (lide_c_rec_actor_new(
            context->fifos[DSG_FIFO_SND_REC],
            context->fifos[DSG_FIFO_LAE2_REC],
            context->fifos[DSG_FIFO_REC_ADDSCA],
            mutex, cond, contextApp->fifos[FIFO_YSRC_ADD], 1, DSG_ACTOR_REC));
    lide_c_dsg_add_actor(REC,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_REC], DSG_ACTOR_REC);
    
    /* DSG actor addSCA */
    context->actors[DSG_ACTOR_ADDSCA] =(lide_c_actor_context_type *) (lide_c_sca_static_loop_new(
           context->fifos[DSG_FIFO_REC_ADDSCA],
           context->fifos[DSG_FIFO_ADDREF_ADDSCA],
           context->fifos[DSG_FIFO_ADDSCA_ADDREF],
           context->fifos[DSG_FIFO_ADDSCA_SNKREF],
            1, DSG_ACTOR_ADDSCA));
    lide_c_dsg_add_actor(SCA_STATIC_LOOP,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_ADDSCA], DSG_ACTOR_ADDSCA);
    
    /* DSC actor addRef */
    context->actors[DSG_ACTOR_ADDREF] = (lide_c_actor_context_type *)(lide_c_ref_actor_new(
           context->fifos[DSG_FIFO_ADDSCA_ADDREF],
           context->fifos[DSG_FIFO_ADDREF_ADDSCA], DSG_ACTOR_ADDREF,
            contextApp->actors[ACTOR_ADD], 0, 0, NULL, NULL, NULL ));
    lide_c_dsg_add_actor(REF,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_ADDREF], DSG_ACTOR_ADDREF);
    
    /* DSG actor sinkREF */
    context->actors[DSG_ACTOR_SNKREF] = (lide_c_actor_context_type *)(lide_c_ref_actor_new(
       context->fifos[DSG_FIFO_ADDSCA_SNKREF],
       context->fifos[DSG_FIFO_SNKREF_LAE2], DSG_ACTOR_SNKREF,
        contextApp->actors[ACTOR_SINK], 0, 0, NULL, NULL, NULL ));
    lide_c_dsg_add_actor(REF,(lide_c_graph_context_type *) context, context->actors[DSG_ACTOR_SNKREF], DSG_ACTOR_SNKREF);
    
    context->scheduler = (lide_c_graph_scheduler_ftype)
            lide_c_add_shched2_scheduler;
    
    /* Delay */
    lide_c_fifo_basic_write_advance((lide_c_fifo_basic_pointer)context->fifos[DSG_FIFO_SND_LAE1]);
    lide_c_fifo_basic_write_advance((lide_c_fifo_basic_pointer)context->fifos[DSG_FIFO_SNKREF_LAE2]);
  
    add_shched2->sdsg = context;
   
    return add_shched2;
}



void lide_c_add_shched2_scheduler(lide_c_add_shched2_context_type *graph, int thread_ind, int thread_num){
    
    int start_ind;
    int end_ind;
    start_ind = graph->first_actors_index[thread_ind];
    if (thread_ind != thread_num -1){
        end_ind =graph->first_actors_index[ thread_ind+1 ] - 1;
    }else{
        end_ind = graph->dsg_count -1;
    }
    
    /*simple scheduler*/
    //lide_c_util_cdsg_scheduler( graph->sdsg->actors, start_ind, end_ind, graph->sdsg->descriptors );
    /*dtt scheduler*/
    lide_c_util_dsg_optimized_scheduler( graph->sdsg, start_ind, end_ind-start_ind + 1);
    
}
