/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lide_c_param_spec.h"

#define NAME_LENGTH 20

struct _lide_c_param_spec_context_struct {
#include "lide_c_graph_context_type_common.h"

int mode;

};

lide_c_param_spec_context_type *lide_c_param_spec_new(
        lide_c_actor_context_type *init, lide_c_actor_context_type *subinit,
	   lide_c_actor_context_type *body){
    
	int i;
    
    lide_c_param_spec_context_type *context = NULL;

    context = lide_c_util_malloc(sizeof(lide_c_param_spec_context_type));
	context->actor_count = ACTOR_COUNT_PARAM;
	context->actors = (lide_c_actor_context_type **)lide_c_util_malloc(
                    context->actor_count * sizeof(lide_c_actor_context_type *));
    context->descriptors = (char **)lide_c_util_malloc(
                        (context->actor_count) * sizeof(char *));
    for(i = 0; i < (context->actor_count); i++){
        context->descriptors[i] = (char *)lide_c_util_malloc(
                        NAME_LENGTH * sizeof(char));
    }
    strcpy(context->descriptors[INIT],"initial");
    strcpy(context->descriptors[SUBINIT],"subinitial");
    strcpy(context->descriptors[BODY],"body");

	context->actors[INIT] = init;
	context->actors[SUBINIT] = subinit;
	context->actors[BODY] = body;
	
	if(context->actors[INIT] != NULL) {
        context->mode = LIDE_C_PARAM_SPEC_MODE_HP_SETUP;
	}
    else if (context->actors[SUBINIT] != NULL) {
        context->mode = LIDE_C_PARAM_SPEC_MODE_P_SETUP;
	}
	context->scheduler = (lide_c_graph_scheduler_ftype) 
	                                    lide_c_param_spec_scheduler;
										
    return context;
}

void lide_c_param_spec_scheduler(lide_c_param_spec_context_type *context){
		// The code here could be reduced to avoid code redundant later.	
		switch (context->mode) {
        case LIDE_C_PARAM_SPEC_MODE_HP_SETUP:
		    lide_c_util_guarded_execution(context->actors[INIT], 
	            context->descriptors[INIT]);
			if (context->actors[SUBINIT] != NULL) {
			    lide_c_util_guarded_execution(context->actors[SUBINIT], 
	            context->descriptors[SUBINIT]);
		        lide_c_util_guarded_execution(context->actors[BODY], 
	            context->descriptors[BODY]);
			context->mode = LIDE_C_PARAM_SPEC_MODE_P_SETUP;
            }
            else {
				context->mode = LIDE_C_PARAM_SPEC_MODE_BODY;
			}			
	        break;
		case LIDE_C_PARAM_SPEC_MODE_P_SETUP:
            lide_c_util_guarded_execution(context->actors[SUBINIT], 
	            context->descriptors[SUBINIT]);
		    lide_c_util_guarded_execution(context->actors[BODY], 
	            context->descriptors[BODY]);
			context->mode = LIDE_C_PARAM_SPEC_MODE_P_SETUP;  
		break;
		case LIDE_C_PARAM_SPEC_MODE_BODY:
		     lide_c_util_guarded_execution(context->actors[BODY], 
	            context->descriptors[BODY]);
			context->mode = LIDE_C_PARAM_SPEC_MODE_BODY;
        default:
            break;
    }			
}

void lide_c_param_spec_terminate(
        lide_c_param_spec_context_type *context){
    free(context->actors);
    free(context->descriptors);
    free(context);
    
    return;

}



