/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_add_decouple.h"
#include "lide_c_util.h"
#include "lide_c_graph.h"

#define MAX_FIFO_COUNT  3

struct _lide_c_add_de_context_struct {
#include "lide_c_actor_context_type_common.h"

    /* Actor interface ports. */
    // lide_c_fifo_basic_pointer in1;
    // lide_c_fifo_basic_pointer in2;
    // lide_c_fifo_basic_pointer out;
};

lide_c_add_de_context_type *lide_c_add_de_new(/*lide_c_fifo_pointer in1,
        lide_c_fifo_pointer in2, lide_c_fifo_pointer out, */int index) {

    lide_c_add_de_context_type *context = NULL;

    context = lide_c_util_malloc(sizeof(lide_c_add_de_context_type));
    context->mode = LIDE_C_ADD_DE_MODE_PROCESS;
    context->enable = (lide_c_actor_enable_ftype)lide_c_add_de_enable;
    context->invoke = (lide_c_actor_invoke_ftype)lide_c_add_de_invoke;
    context->reset = (lide_c_actor_reset_ftype)lide_c_add_de_reset;
    // context->in1 = (lide_c_fifo_basic_pointer)in1;
    // context->in2 = (lide_c_fifo_basic_pointer)in2;
    // context->out = (lide_c_fifo_basic_pointer)out;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)lide_c_add_de_connect;
    context->max_port_count = MAX_FIFO_COUNT;
    
    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context, 
        FIFO_COUNT, &context->in1, &context->in2, &context->out);
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    // lide_c_actor_add_de_portrefs((lide_c_actor_context_type *)context, 
    //                             (lide_c_fifo_pointer *)&context->in1);    
    // lide_c_actor_add_de_portrefs((lide_c_actor_context_type *)context, 
    //                             (lide_c_fifo_pointer *)&context->in2);  
    // lide_c_actor_add_de_portrefs((lide_c_actor_context_type *)context, 
    //                             (lide_c_fifo_pointer *)&context->out);  
                                
    return context;
}

boolean lide_c_add_de_enable(lide_c_add_de_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_ADD_DE_MODE_PROCESS:
        
        /*
        result = (context->in1->get_population(context->in1) >= 1) &&
                (context->in2->get_population(context->in2) >= 1) &&
                ((context->out->get_population(context->out) < 
                context->out->get_capacity(context->out)));
        */
        
        result = (lide_c_fifo_basic_population(
                (lide_c_fifo_basic_pointer)*(context->portrefs[0])) >= 1) 
                && (lide_c_fifo_basic_population(
                (lide_c_fifo_basic_pointer)*(context->portrefs[1])) >= 1) 
                && ((lide_c_fifo_basic_population(
                (lide_c_fifo_basic_pointer)*(context->portrefs[2])) < 
                lide_c_fifo_basic_capacity(
                (lide_c_fifo_basic_pointer)*(context->portrefs[2]))));
        
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.

*******************************************************************************/
void lide_c_add_de_invoke(lide_c_add_de_context_type *context) {
    // printf("add decouple actor invoked\n");
    int value1 = 0;
    int value2 = 0;
    int sum = 0;

    switch (context->mode) {
    case LIDE_C_ADD_DE_MODE_PROCESS:
        //lide_c_fifo_basic_read(context->in1, &value1);
        lide_c_fifo_basic_read(
            (lide_c_fifo_basic_pointer)*(context->portrefs[0]), &value1);
        //lide_c_fifo_basic_read(context->in2, &value2);
        lide_c_fifo_basic_read(
            (lide_c_fifo_basic_pointer)*(context->portrefs[1]), &value2);
        sum = value1 + value2;
        //lide_c_fifo_basic_write(context->out, &sum);
        lide_c_fifo_basic_write(
            (lide_c_fifo_basic_pointer)*(context->portrefs[2]), &sum);
        context->mode = LIDE_C_ADD_DE_MODE_PROCESS;
        break;
    default:
        context->mode = LIDE_C_ADD_DE_MODE_PROCESS;
        break;
    }
}

void lide_c_add_de_terminate(lide_c_add_de_context_type *context) {
    free(context);
}

void lide_c_add_de_reset(lide_c_add_de_context_type *context){
    
    context->mode = LIDE_C_ADD_DE_MODE_PROCESS;
    return;
}
void lide_c_add_de_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    /* input 2*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 1;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 2;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);
    return;
} 
