/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "lide_c_file_sink_decouple.h"
#include "lide_c_util.h"
#include "lide_c_graph.h"

#define MAX_FIFO_COUNT 1


struct _lide_c_file_sink_decouple_context_struct {
#include "lide_c_actor_context_type_common.h"
    FILE *fp;  
    char *file;
    // lide_c_fifo_basic_pointer in;
};


lide_c_file_sink_decouple_context_type *lide_c_file_sink_decouple_new(char *file/*,
        lide_c_fifo_pointer in*/, int index) {

    lide_c_file_sink_decouple_context_type *context = NULL;

    context = lide_c_util_malloc(sizeof(lide_c_file_sink_decouple_context_type));
    context->invoke = 
            (lide_c_actor_invoke_ftype)lide_c_file_sink_decouple_invoke;
    context->enable = 
            (lide_c_actor_enable_ftype)lide_c_file_sink_decouple_enable;
    context->reset = 
            (lide_c_actor_reset_ftype)lide_c_file_sink_decouple_reset;
    context->file = file;
    context->fp = lide_c_util_fopen((const char *)context->file, "w");
    context->mode = LIDE_C_FILE_SINK_DECOUPLE_MODE_READ;
    // context->in = (lide_c_fifo_basic_pointer)in;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)lide_c_file_sink_decouple_connect;
    context->max_port_count = MAX_FIFO_COUNT;
    
    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context, 
        FIFO_COUNT, &context->in);
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    // lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                // (lide_c_fifo_pointer *)&context->in);
    
    return context;
}

boolean lide_c_file_sink_decouple_enable(lide_c_file_sink_decouple_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_FILE_SINK_DECOUPLE_MODE_READ:
        //result = lide_c_fifo_basic_population(context->in) >= 1;
        result = lide_c_fifo_basic_population(
                    (lide_c_fifo_basic_pointer)*(context->portrefs[0])) >= 1;
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

void lide_c_file_sink_decouple_invoke(lide_c_file_sink_decouple_context_type *context) {
    int value = 0;

    switch (context->mode) {
    case LIDE_C_FILE_SINK_DECOUPLE_MODE_READ:
        //lide_c_fifo_basic_read(context->in, &value);
        lide_c_fifo_basic_read(
            (lide_c_fifo_basic_pointer)*(context->portrefs[0]), &value);
        fprintf(context->fp, "%d\n", value);
        context->mode = LIDE_C_FILE_SINK_DECOUPLE_MODE_READ;
        break;
    default:
        context->mode = LIDE_C_FILE_SINK_DECOUPLE_MODE_READ;
        break;
    }
}

void lide_c_file_sink_decouple_terminate(lide_c_file_sink_decouple_context_type *context) {
    fclose(context->fp);
    free(context);
}

void lide_c_file_sink_decouple_reset(lide_c_file_sink_decouple_context_type *context){
    fclose(context->fp);
    context->fp = lide_c_util_fopen((const char *)context->file, "w");
    context->mode = LIDE_C_FILE_SINK_DECOUPLE_MODE_READ;
    return;
}


void lide_c_file_sink_decouple_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){

    int port_index;
    int direction;
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    return;


}
