/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_file_source.h"
#include "lide_c_graph.h"
#include "lide_c_util.h"

#define MAX_FIFO_COUNT  1

struct _lide_c_file_source_context_struct {
#include "lide_c_actor_context_type_common.h"
    FILE *fp; 
    char *file;
    int data;
    lide_c_fifo_basic_pointer out;
};


lide_c_file_source_context_type *lide_c_file_source_new(char *file,
        lide_c_fifo_pointer out, int index) {

    lide_c_file_source_context_type *context = NULL;
    //printf("file source init\n");
    context = lide_c_util_malloc(sizeof(lide_c_file_source_context_type));
    context->enable =
            (lide_c_actor_enable_ftype)lide_c_file_source_enable;
    context->invoke =
            (lide_c_actor_invoke_ftype)lide_c_file_source_invoke;
    context->reset =
            (lide_c_actor_invoke_ftype)lide_c_file_source_reset;
    context->file = file;
    context->fp = lide_c_util_fopen((const char *)context->file, "r");
    if (fscanf(context->fp, "%d", &context->data) != 1) { 
            /* End of input */
        context->mode = LIDE_C_FILE_SOURCE_MODE_INACTIVE;
        context->data = 0;
        
    }else{
        context->mode = LIDE_C_FILE_SOURCE_MODE_WRITE;
    }
    
    context->out = (lide_c_fifo_basic_pointer)out;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)lide_c_file_source_connect;
    context->max_port_count = MAX_FIFO_COUNT;

    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context, 
        FIFO_COUNT, &context->out);
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out);
    
    return context;
}

boolean lide_c_file_source_enable(lide_c_file_source_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_FILE_SOURCE_MODE_WRITE:
    /*
        result = (lide_c_fifo_basic_population(context->out) < 
                lide_c_fifo_basic_capacity(context->out));
   */
        result = (lide_c_fifo_basic_population( 
                (lide_c_fifo_basic_pointer)*(context->portrefs[0])) < 
                lide_c_fifo_basic_capacity( 
                (lide_c_fifo_basic_pointer)*(context->portrefs[0])));
   
  
    break;
    case LIDE_C_FILE_SOURCE_MODE_INACTIVE:
        result = FALSE;
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

void lide_c_file_source_invoke(lide_c_file_source_context_type *context) {
    switch (context->mode) {
    case LIDE_C_FILE_SOURCE_MODE_WRITE:
        //lide_c_fifo_basic_write(context->out, &context->data);
          //  printf("file source");
        lide_c_fifo_basic_write( 
            (lide_c_fifo_basic_pointer)*(context->portrefs[0]), 
            &context->data);
        if (fscanf(context->fp, "%d", &context->data) != 1) { 
            /* End of input */
            context->mode = LIDE_C_FILE_SOURCE_MODE_INACTIVE;
        }else{
            context->mode = LIDE_C_FILE_SOURCE_MODE_WRITE;;
        }
        break;
    case LIDE_C_FILE_SOURCE_MODE_INACTIVE:
        context->mode = LIDE_C_FILE_SOURCE_MODE_INACTIVE;
        break;
    default:
        context->mode = LIDE_C_FILE_SOURCE_MODE_INACTIVE;
        break;
    }
}

void lide_c_file_source_terminate(lide_c_file_source_context_type *context) {
    fclose(context->fp);
    free(context);
}

void lide_c_file_source_reset(lide_c_file_source_context_type *context){
    fclose(context->fp);
    context->fp = lide_c_util_fopen((const char *)context->file, "r");
    if (fscanf(context->fp, "%d", &context->data) != 1) { 
        context->mode = LIDE_C_FILE_SOURCE_MODE_INACTIVE;
        context->data = 0;
        
    }else{
        context->mode = LIDE_C_FILE_SOURCE_MODE_WRITE;
    }
    return;
}


void lide_c_file_source_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    
    int port_index;
    int direction;
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    return;
}


