#ifndef _lide_c_file_source_decouple_h
#define _lide_c_file_source_decouple_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include "lide_c_actor.h"
#include "lide_c_fifo.h"
#include "lide_c_fifo_basic.h"
#include "lide_c_graph_def.h"

/* Actor modes */
#define LIDE_C_FILE_SOURCE_DECOUPLE_MODE_WRITE        1
#define LIDE_C_FILE_SOURCE_DECOUPLE_MODE_INACTIVE     2 

/*******************************************************************************
TYPE DEFINITIONS
*******************************************************************************/

/* Structure and pointer types associated with file source objects. */
struct _lide_c_file_source_decouple_context_struct;
typedef struct _lide_c_file_source_decouple_context_struct
        lide_c_file_source_decouple_context_type;

/*******************************************************************************
INTERFACE FUNCTIONS
*******************************************************************************/

/*****************************************************************************
Construct function of the lide_c_file_source_decouple actor. Create a new
lide_c_file_source_decouple with the specified file pointer, and the specified output
FIFO pointer. Data format in the input file should be all integers.
The only one port is for the output port.
*****************************************************************************/
lide_c_file_source_decouple_context_type *lide_c_file_source_decouple_new(char *file,
        /*lide_c_fifo_pointer out, */int index);

/*****************************************************************************
Enable function of the lide_c_file_source_decouple actor.
*****************************************************************************/
boolean lide_c_file_source_decouple_enable(lide_c_file_source_decouple_context_type *context); 

/*****************************************************************************
Invoke function of the lide_c_file_source_decouple actor.
*****************************************************************************/
void lide_c_file_source_decouple_invoke(lide_c_file_source_decouple_context_type *context); 

/*****************************************************************************
Terminate function of the lide_c_file_source_decouple actor.
*****************************************************************************/
void lide_c_file_source_decouple_terminate(lide_c_file_source_decouple_context_type *context);

/*****************************************************************************
Reset function of the lide_c_file_source_decouple actor.
*****************************************************************************/
void lide_c_file_source_decouple_reset(lide_c_file_source_decouple_context_type *context);

/*****************************************************************************
Add connection function of the lide_c_file_source_decouple actor.
*****************************************************************************/
void lide_c_file_source_decouple_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph);  

#endif
