/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
 #include <sys/time.h>

#include "lide_c_rec_actor.h"
#include "lide_c_util.h"
#include "lide_c_graph.h"
#define MAX_FIFO_COUNT 3

struct _lide_c_rec_actor_context_struct {
#include "lide_c_cdsg_actor_context_type_common.h"  

/* Actor interface ports. */
lide_c_fifo_basic_pointer in_ipc; /*Input IPC edge*/
lide_c_fifo_unit_size_pointer in_pc; /*Input PC edge*/
lide_c_fifo_unit_size_pointer out_pc; /*Output PC edge*/
pthread_mutex_t *mutex;
pthread_cond_t *cond;
lide_c_fifo_basic_pointer fifo;
int block_size;
};

lide_c_rec_actor_context_type *lide_c_rec_actor_new(
        lide_c_fifo_pointer in_ipc, lide_c_fifo_pointer in_pc, 
        lide_c_fifo_pointer out_pc, pthread_mutex_t *mutex,
        pthread_cond_t *cond, lide_c_fifo_pointer fifo,
        int block_size, int index){
    lide_c_rec_actor_context_type *context = NULL;

    context = (lide_c_rec_actor_context_type *)lide_c_util_malloc(
                sizeof(lide_c_rec_actor_context_type));
    context->mode = LIDE_C_REC_ACTOR_MODE_PROCESS;
    context->enable = (lide_c_actor_enable_ftype)
                        lide_c_rec_actor_enable;
    context->invoke = (lide_c_actor_invoke_ftype)
                        lide_c_rec_actor_invoke;
    
    context->in_pc = (lide_c_fifo_unit_size_pointer)in_pc;
    context->in_ipc = (lide_c_fifo_basic_pointer)in_ipc;
    context->out_pc = (lide_c_fifo_unit_size_pointer)out_pc;
    context->last_fifo = (lide_c_fifo_unit_size_pointer)out_pc;

    context->mutex = mutex;
    context->cond = cond;
    context->fifo = (lide_c_fifo_basic_pointer)fifo;
    context->block_size = block_size;

    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)
                        lide_c_rec_actor_connect;
    context->max_port_count = MAX_FIFO_COUNT;
    
    /* Initialize portrefs*/
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in_ipc);    
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in_pc);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out_pc);   
    
    return context;
}

boolean lide_c_rec_actor_enable(lide_c_rec_actor_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_REC_ACTOR_MODE_PROCESS:
        result = (lide_c_fifo_unit_size_population(context->in_pc) >= 1)
                && (lide_c_fifo_unit_size_population(context->out_pc) < 
                    lide_c_fifo_unit_size_capacity(context->out_pc));
        break;
    case LIDE_C_REC_ACTOR_MODE_ERROR:
        result = FALSE;
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.
*******************************************************************************/

/* In the next step, use dlpp to avoid if checkings in invoke function*/
void lide_c_rec_actor_invoke(lide_c_rec_actor_context_type *context) {

    switch (context->mode) {
    case LIDE_C_REC_ACTOR_MODE_PROCESS:
           // printf("REC actor\n");
           // printf("%d, %d\n",lide_c_fifo_basic_population(context->in_ipc),lide_c_fifo_basic_population(context->in_pc) );
       pthread_mutex_lock(context->mutex);
       // printf("REC actor mutex lock\n\n");
        while((lide_c_fifo_basic_population(context->in_ipc) < 1)
             || !((lide_c_fifo_basic_population(context->fifo) >= 
                context->block_size))){
            //printf("REC actor mutex wait\n");
            pthread_cond_wait(context->cond, context->mutex);
            
        }
        
        pthread_mutex_unlock(context->mutex);
         //   printf("REC actor mutex exe\n");
        lide_c_fifo_unit_size_read_advance(context->in_pc);
        //    printf("REC unit_size_read_advance\n");
        lide_c_fifo_basic_read_advance(context->in_ipc);
        //    printf("REC unit_size_write_advance\n");
        lide_c_fifo_unit_size_write_advance(context->out_pc);
        break;
    default:
        context->mode = LIDE_C_REC_ACTOR_MODE_PROCESS;
        break;
    }
}

void lide_c_rec_actor_terminate(lide_c_rec_actor_context_type *context) {
    free(context);
}

void lide_c_rec_actor_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;

    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    /* output 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 1;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    /* output 2*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 2;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    return;
}

lide_c_fifo_unit_size_pointer lide_c_rec_actor_dsg_fifo_output(
        lide_c_rec_actor_context_type *context){
    return context->last_fifo;
}

