/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_ref_actor.h"
#include "lide_c_util.h"
#include "lide_c_graph.h"
#include "lide_c_ref_pre_post_functions.h"
#define MAX_FIFO_COUNT 2

lide_c_ref_actor_context_type *lide_c_ref_actor_new(
        //lide_c_fifo_unit_size_pointer in, lide_c_fifo_unit_size_pointer out, 
        lide_c_fifo_pointer in, lide_c_fifo_pointer out, 
        int index,
        lide_c_actor_context_type *ref_context,
        int selected_input_count, int selected_output_count, 
        lide_c_ref_actor_preA_ftype preA, 
        lide_c_ref_actor_postA_ftype postA,
        lide_c_ref_actor_exeA_ftype exeA) {

    lide_c_ref_actor_context_type *context = NULL;

    context = (lide_c_ref_actor_context_type *)lide_c_util_malloc(
                sizeof(lide_c_ref_actor_context_type));
    context->mode = LIDE_C_REF_ACTOR_MODE_PROCESS;
    context->enable = (lide_c_actor_enable_ftype)
                        lide_c_ref_actor_enable;
    context->invoke = (lide_c_actor_invoke_ftype)
                        lide_c_ref_actor_invoke;
    
    context->ref_context = ref_context;

    context->preA = (lide_c_ref_actor_preA_ftype)preA;
    context->postA = (lide_c_ref_actor_postA_ftype)postA;
    context->exeA = (lide_c_ref_actor_exeA_ftype)exeA;
    
    context->selected_input_count = selected_input_count;
    context->selected_output_count = selected_output_count;
    if (context->selected_input_count < 0 || 
        context->selected_output_count < 0){
        context->mode = LIDE_C_REF_ACTOR_MODE_ERROR;
    }else{
        context->selected_app_gr_inputs = 
            (lide_c_fifo_pointer *)lide_c_util_malloc(
            sizeof(lide_c_fifo_pointer)*context->selected_input_count);
        context->selected_app_gr_outputs = 
            (lide_c_fifo_pointer *)lide_c_util_malloc(
            sizeof(lide_c_fifo_pointer)*context->selected_output_count);
    }
    
    
    context->in = (lide_c_fifo_unit_size_pointer)in;
    context->out = (lide_c_fifo_unit_size_pointer)out;
    context->last_fifo = context->out;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)
                        lide_c_ref_actor_connect;
    context->max_port_count = MAX_FIFO_COUNT;
    
    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context, 
        FIFO_COUNT, &context->in, &context->out);
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in);    
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out);  
    
    return context;
}


boolean lide_c_ref_actor_enable(lide_c_ref_actor_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_REF_ACTOR_MODE_PROCESS:
        result = (lide_c_fifo_unit_size_population(context->in) >= 1) &&
                ((lide_c_fifo_unit_size_population(context->out) < 
                lide_c_fifo_unit_size_capacity(context->out)));
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.
*******************************************************************************/

/* In the next step, use dlpp to avoid if checkings in invoke function*/
void lide_c_ref_actor_invoke(lide_c_ref_actor_context_type *context) {
    //printf("start ref\n");
    switch (context->mode) {
    case LIDE_C_REF_ACTOR_MODE_PROCESS:
        lide_c_fifo_unit_size_read(context->in, &context->common_state);
        if(context->preA != NULL)
            context->preA(context);
        
        if(context->exeA != NULL){
            context->exeA(context);
        }else{
            if (context->ref_context->enable(context->ref_context)) {
                context->ref_context->invoke(context->ref_context);
            }
        }
        
        if(context->postA != NULL){
            //printf("do post\n");
            context->postA(context);
            
        }
        
        lide_c_fifo_unit_size_write(context->out, &context->common_state);
        context->mode = LIDE_C_REF_ACTOR_MODE_PROCESS;
            //printf("end ref\n");
        break;
    default:
        context->mode = LIDE_C_REF_ACTOR_MODE_PROCESS;
        break;
    }
}

void lide_c_ref_actor_terminate(lide_c_ref_actor_context_type *context) {
    if(context->selected_app_gr_inputs != NULL){
        free(context->selected_app_gr_inputs);
    }
    if(context->selected_app_gr_outputs != NULL){
        free(context->selected_app_gr_outputs);
    }
    free(context);
}


lide_c_fifo_unit_size_pointer lide_c_ref_actor_dsg_fifo_output(
        lide_c_ref_actor_context_type *context){
    return context->last_fifo;
}



void lide_c_ref_actor_set_preA_function(lide_c_ref_actor_context_type *context, 
    lide_c_ref_actor_preA_ftype preA){
    context->preA = preA;
    return;
}

int lide_c_ref_actor_get_preA_function(lide_c_ref_actor_context_type *context){
    return context->mode;
}
    
void lide_c_ref_actor_set_postA_function(lide_c_ref_actor_context_type *context, 
    lide_c_ref_actor_postA_ftype postA){
    context->postA = postA;
    return;
}
    
int lide_c_ref_actor_get_postA_function(lide_c_ref_actor_context_type *context){
    return context->mode;
}

void lide_c_ref_actor_set_exeA_function(lide_c_ref_actor_context_type *context, 
    lide_c_ref_actor_exeA_ftype exeA){
    context->exeA = exeA;
    return;
}
    
int lide_c_ref_actor_get_exeA_function(lide_c_ref_actor_context_type *context){
    return context->mode;
}


    
void lide_c_ref_actor_set_selected_input_count(
    lide_c_ref_actor_context_type *context, int selected_input_count){
    context->selected_input_count = selected_input_count;
    if (context->selected_input_count < 0){
        context->mode = LIDE_C_REF_ACTOR_MODE_ERROR;
    }else{
        context->mode = LIDE_C_REF_ACTOR_MODE_PROCESS;
        if(context->selected_app_gr_inputs != NULL){
            free(context->selected_app_gr_inputs);
        }
        context->selected_app_gr_inputs = 
            (lide_c_fifo_pointer *)lide_c_util_malloc(
            sizeof(lide_c_fifo_pointer)*context->selected_input_count);
        
    }
    
}

int lide_c_ref_actor_get_selected_input_count(
    lide_c_ref_actor_context_type *context){
    return context->selected_input_count;
}
    
void lide_c_ref_actor_set_selected_output_count(
    lide_c_ref_actor_context_type *context, int selected_output_count){
    context->selected_output_count = selected_output_count;
    if (context->selected_output_count < 0){
        context->mode = LIDE_C_REF_ACTOR_MODE_ERROR;
    }else{
        context->mode = LIDE_C_REF_ACTOR_MODE_PROCESS;
        if(context->selected_app_gr_outputs != NULL){
            free(context->selected_app_gr_outputs);
        }
        context->selected_app_gr_outputs = 
            (lide_c_fifo_pointer *)lide_c_util_malloc(
            sizeof(lide_c_fifo_pointer)*context->selected_output_count);
        
    }
        
}

int lide_c_ref_actor_get_selected_output_count(
    lide_c_ref_actor_context_type *context){
    return context->selected_output_count;
}
    
void lide_c_ref_actor_set_selected_app_gr_inputs(
    lide_c_ref_actor_context_type *context, lide_c_fifo_pointer input, 
    int index){
    context->selected_app_gr_inputs[index] = input;
    return;    
}

lide_c_fifo_pointer lide_c_ref_actor_get_selected_app_gr_inputs(
    lide_c_ref_actor_context_type *context, int index){
    return(context->selected_app_gr_inputs[index]);
}

void lide_c_ref_actor_set_selected_app_gr_outputs(
    lide_c_ref_actor_context_type *context, lide_c_fifo_pointer output, 
    int index){
    context->selected_app_gr_outputs[index] = output;
    return;       
}

lide_c_fifo_pointer lide_c_ref_actor_get_selected_app_gr_outputs(
    lide_c_ref_actor_context_type *context, int index){
    return(context->selected_app_gr_outputs[index]);
}

void lide_c_ref_actor_set_index(lide_c_ref_actor_context_type *context, 
    int index){
    context->index = index;
    return;
} 

int lide_c_ref_actor_get_index(lide_c_ref_actor_context_type *context){
    return context->index;
} 


void lide_c_ref_actor_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 1;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    return;
}
