#ifndef _lide_c_ref_actor_h
#define _lide_c_ref_actor_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include "lide_c_actor.h"
#include "lide_c_dsg_actor.h"
#include "lide_c_fifo.h"
#include "lide_c_basic.h"
#include "lide_c_graph_def.h"
#include "lide_c_fifo_unit_size.h"


/* Actor modes */
#define LIDE_C_REF_ACTOR_MODE_PROCESS  0 
#define LIDE_C_REF_ACTOR_MODE_ERROR    1 


/*******************************************************************************
TYPE DEFINITIONS
*******************************************************************************/

/* Structure and pointer types associated with ref_actor objects. */
struct _lide_c_ref_actor_context_struct;
typedef struct _lide_c_ref_actor_context_struct lide_c_ref_actor_context_type;

typedef void (*lide_c_ref_actor_preA_ftype) 
        (lide_c_ref_actor_context_type *context);
        
typedef void (*lide_c_ref_actor_postA_ftype) 
        (lide_c_ref_actor_context_type *context);
        
typedef void (*lide_c_ref_actor_exeA_ftype) 
        (lide_c_ref_actor_context_type *context);
        
typedef void (*lide_c_ref_actor_dsg_fifo_output_ftype) 
        (lide_c_ref_actor_context_type *context);

struct _lide_c_ref_actor_context_struct {
#include "lide_c_ref_actor_context_type_common.h"  
};

        
/*******************************************************************************
INTERFACE FUNCTIONS
*******************************************************************************/

lide_c_ref_actor_context_type *lide_c_ref_actor_new(
        //lide_c_fifo_unit_size_pointer in, lide_c_fifo_unit_size_pointer out, 
        lide_c_fifo_pointer in, lide_c_fifo_pointer out, 
        int index,
        lide_c_actor_context_type *ref_context,
        int selected_input_count, int selected_output_count, 
        lide_c_ref_actor_preA_ftype preA, 
        lide_c_ref_actor_postA_ftype postA,
        lide_c_ref_actor_exeA_ftype exeA);

boolean lide_c_ref_actor_enable(lide_c_ref_actor_context_type *context); 

void lide_c_ref_actor_invoke(lide_c_ref_actor_context_type *context); 

void lide_c_ref_actor_terminate(lide_c_ref_actor_context_type *context);

/* Get the only DSG output FIFO in RA*/
lide_c_fifo_unit_size_pointer lide_c_ref_actor_dsg_fifo_output(
        lide_c_ref_actor_context_type *context);


/* Set and get function for parameters*/
void lide_c_ref_actor_set_preA_function(
    lide_c_ref_actor_context_type *context, 
    lide_c_ref_actor_preA_ftype preA);

int lide_c_ref_actor_get_preA_function(
    lide_c_ref_actor_context_type *context);
    
void lide_c_ref_actor_set_postA_function(
    lide_c_ref_actor_context_type *context, 
    lide_c_ref_actor_postA_ftype postA);
    
int lide_c_ref_actor_get_postA_function(
    lide_c_ref_actor_context_type *context);
    
void lide_c_ref_actor_set_exeA_function(
    lide_c_ref_actor_context_type *context, 
    lide_c_ref_actor_exeA_ftype exeA);

int lide_c_ref_actor_get_exeA_function(
    lide_c_ref_actor_context_type *context);
    
void lide_c_ref_actor_set_selected_input_count(
    lide_c_ref_actor_context_type *context, int selected_input_count);

int lide_c_ref_actor_get_selected_input_count(
    lide_c_ref_actor_context_type *context);
    
void lide_c_ref_actor_set_selected_output_count(
    lide_c_ref_actor_context_type *context, int selected_output_count);

int lide_c_ref_actor_get_selected_output_count(
    lide_c_ref_actor_context_type *context);
    
void lide_c_ref_actor_set_selected_app_gr_inputs(
    lide_c_ref_actor_context_type *context, lide_c_fifo_pointer input, 
    int index);

lide_c_fifo_pointer lide_c_ref_actor_get_selected_app_gr_inputs(
    lide_c_ref_actor_context_type *context, int index);

void lide_c_ref_actor_set_selected_app_gr_outputs(
    lide_c_ref_actor_context_type *context, lide_c_fifo_pointer output, 
    int index);

lide_c_fifo_pointer lide_c_ref_actor_get_selected_app_gr_outputs(
    lide_c_ref_actor_context_type *context, int index);


void lide_c_ref_actor_set_index(lide_c_ref_actor_context_type *context, 
    int index);  

int lide_c_ref_actor_get_index(lide_c_ref_actor_context_type *context);  


void lide_c_ref_actor_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph);   
    
#endif
