/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_cdsg_static_loop.h"
#include "lide_c_graph.h"
#include "lide_c_util.h"

#define MAX_FIFO_COUNT 4

struct _lide_c_cdsg_static_loop_context_struct {
#include "lide_c_sca_context_type_common.h" 

/* sub mode*/
int process_submode;

/* FIFO ports. */
lide_c_fifo_unit_size_pointer in1; /*loop_start*/
lide_c_fifo_unit_size_pointer in2; /*next_iteration*/
lide_c_fifo_unit_size_pointer out1; /*iteration_start*/
lide_c_fifo_unit_size_pointer out2; /*loop_end*/

/* Number of rest iterations */
int *current_iteration;
/* Total number of iterations in the loop*/
int iteration_count;

/* Control data received from input1 */
int in1_ctrl;
};

lide_c_cdsg_static_loop_context_type *lide_c_cdsg_static_loop_new(
        //lide_c_fifo_unit_size_pointer in1, 
        lide_c_fifo_pointer in1, 
        //lide_c_fifo_unit_size_pointer in2, 
        lide_c_fifo_pointer in2, 
        //lide_c_fifo_unit_size_pointer out1, 
        lide_c_fifo_pointer out1, 
        //lide_c_fifo_unit_size_pointer out2,
        lide_c_fifo_pointer out2,
        int iteration_count, int *current_iteration, int index){

    lide_c_cdsg_static_loop_context_type *context = NULL;

    context = (lide_c_cdsg_static_loop_context_type *)lide_c_util_malloc(
                sizeof(lide_c_cdsg_static_loop_context_type));
                
    context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_WAIT;
    context->process_submode = LIDE_C_CDSG_STATIC_LOOP_SUBMODE_PROCESS_A;
    
    context->enable = (lide_c_actor_enable_ftype)
                        lide_c_cdsg_static_loop_enable;
    context->invoke = (lide_c_actor_invoke_ftype)
                        lide_c_cdsg_static_loop_invoke;         
  
    context->in1 = (lide_c_fifo_unit_size_pointer)in1;
    context->in2 = (lide_c_fifo_unit_size_pointer)in2;
    context->out1 = (lide_c_fifo_unit_size_pointer)out1;
    context->out2 = (lide_c_fifo_unit_size_pointer)out2;
    context->last_fifo = NULL;
    
    context->current_iteration = current_iteration;
    *(context->current_iteration) = 0;
    context->iteration_count = iteration_count+1;
    context->in1_ctrl = 0;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)
                        lide_c_cdsg_static_loop_connect;
    context->max_port_count = MAX_FIFO_COUNT;
    
    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context, 
        FIFO_COUNT, &context->in1, &context->in2, &context->out1, 
        &context->out2);
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in1); 
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in2);  
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out1); 
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out2); 
                                
    /* iteration_count should be positive */
    if(context->iteration_count < 1){
        context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_ERROR;
    }
    
    return context;
}


boolean lide_c_cdsg_static_loop_enable(lide_c_cdsg_static_loop_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_CDSG_STATIC_LOOP_MODE_PROCESS:
        result = (lide_c_fifo_unit_size_population(context->in2) >= 1) &&
                ((lide_c_fifo_unit_size_population(context->out1) < 
                lide_c_fifo_unit_size_capacity(context->out1)) || 
                (lide_c_fifo_unit_size_population(context->out2) < 
                lide_c_fifo_unit_size_capacity(context->out2)));
        break;
    case LIDE_C_CDSG_STATIC_LOOP_MODE_WAIT:
        result = (lide_c_fifo_unit_size_population(context->in1) >= 1) &&
                (lide_c_fifo_unit_size_population(context->out1) < 
                lide_c_fifo_unit_size_capacity(context->out1));
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.
*******************************************************************************/
void lide_c_cdsg_static_loop_invoke(lide_c_cdsg_static_loop_context_type *context) {

    switch (context->mode) {
    case LIDE_C_CDSG_STATIC_LOOP_MODE_PROCESS:
        lide_c_fifo_unit_size_read_advance(context->in2);
        
        if (*(context->current_iteration) != 0)
        *(context->current_iteration) = *(context->current_iteration)-1;
        
        if(*(context->current_iteration) > 0)
            context->process_submode = LIDE_C_CDSG_STATIC_LOOP_SUBMODE_PROCESS_A;
        else
            context->process_submode = LIDE_C_CDSG_STATIC_LOOP_SUBMODE_PROCESS_B;
        
        if(context->process_submode == LIDE_C_CDSG_STATIC_LOOP_SUBMODE_PROCESS_A){
            
            lide_c_fifo_unit_size_write_advance(context->out1);
            context->last_fifo = context->out1;
            context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_PROCESS;
            
        }else{
            lide_c_fifo_unit_size_write(context->out2, &context->in1_ctrl);
            context->last_fifo = context->out2;
            context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_WAIT;
        }
        printf("iteration count: %d, process: %d\n",*(context->current_iteration),
            context->process_submode);
        break;
    case LIDE_C_CDSG_STATIC_LOOP_MODE_WAIT:
        lide_c_fifo_unit_size_read(context->in1, &context->in1_ctrl);
        *(context->current_iteration) = context->iteration_count;
        lide_c_fifo_unit_size_write_advance(context->out1);
        context->last_fifo = context->out1;
        context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_PROCESS;
        context->process_submode = LIDE_C_CDSG_STATIC_LOOP_SUBMODE_PROCESS_A;
        break;
    default:
        context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_ERROR;
        break;
    }
}

void lide_c_cdsg_static_loop_terminate(lide_c_cdsg_static_loop_context_type *context) {
    free(context);
}

void lide_c_cdsg_static_loop_set_current_iteration(lide_c_cdsg_static_loop_context_type *context, 
    int *current_iteration){
    *(context->current_iteration) = *current_iteration;
    return;
}

int lide_c_cdsg_static_loop_get_current_iteration(lide_c_cdsg_static_loop_context_type *context){
    return *(context->current_iteration);
}

void lide_c_cdsg_static_loop_set_iteration_count(lide_c_cdsg_static_loop_context_type *context, 
    int iteration_count){
    context->iteration_count = iteration_count;
    if(context->mode == LIDE_C_CDSG_STATIC_LOOP_MODE_ERROR){
        context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_WAIT;
    }
        
    if(context->iteration_count < 1){
        context->mode = LIDE_C_CDSG_STATIC_LOOP_MODE_ERROR;
    }
    return;
}

int lide_c_cdsg_static_loop_get_iteration_count(lide_c_cdsg_static_loop_context_type *context){
    return context->iteration_count;
}


void lide_c_cdsg_static_loop_set_index(
lide_c_cdsg_static_loop_context_type *context, int index){
    context->index = index;
    return;
} 

int lide_c_cdsg_static_loop_get_index(
lide_c_cdsg_static_loop_context_type *context){
    return context->index;
} 

void lide_c_cdsg_static_loop_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    /* input 2*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 1;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 2;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 3;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
}
