/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_sca_endcase.h"
#include "lide_c_graph.h"
#include "lide_c_util.h"

#define MAX_FIFO_COUNT  100

struct _lide_c_sca_endcase_context_struct {
#include "lide_c_sca_context_type_common.h" 


/* FIFO ports. */
/* 
    pointer to SCA FI actor's input FIFOs array; number of elements in the 
    array is input_port_count; data type of element in the array is 
    lide_c_fifo_unit_size_pointer
*/
lide_c_fifo_unit_size_pointer *in; 
/* output FIFO*/
lide_c_fifo_unit_size_pointer out1; 

/* total number of input ports in the actor*/
int input_port_count;
/* index of the input port which has tokens storing in the FIFO*/
int pindex;

};

lide_c_sca_endcase_context_type *lide_c_sca_endcase_new(
        //lide_c_fifo_unit_size_pointer *in, 
        lide_c_fifo_pointer *in, 
        //lide_c_fifo_unit_size_pointer out1, 
        lide_c_fifo_pointer out1,
        int input_port_count, int index){
    int i;
    lide_c_sca_endcase_context_type *context = NULL;

    context = (lide_c_sca_endcase_context_type *)lide_c_util_malloc(
                sizeof(lide_c_sca_endcase_context_type));
                
    context->mode = LIDE_C_SCA_ENDCASE_MODE_PROCESS;
    
    context->enable = (lide_c_actor_enable_ftype)lide_c_sca_endcase_enable;
    context->invoke = (lide_c_actor_invoke_ftype)lide_c_sca_endcase_invoke;
   
    context->out1 = (lide_c_fifo_unit_size_pointer)out1;  
    context->input_port_count = input_port_count;
    context->pindex = 0;
    context->last_fifo = NULL;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)
                        lide_c_sca_endcase_connect;
    
    /* pin_num is positive */
    if(context->input_port_count < 1 || context->input_port_count > (MAX_FIFO_COUNT - 1)){
        context->mode = LIDE_C_SCA_ENDCASE_MODE_ERROR;
        context->in = NULL;
        return context;
    }else{
        context->in = (lide_c_fifo_unit_size_pointer *)in;
    }
    context->port_count = 0;
    context->max_port_count = MAX_FIFO_COUNT;
    
    /* Initialize portrefs*/
    /*
    context->port_count = context->input_port_count + 1;
    context->portrefs = (lide_c_fifo_pointer **)lide_c_util_malloc(
                        sizeof(lide_c_fifo_pointer *) * context->port_count);
    
    
    for(i = 0; i<context->input_port_count; i++){
        context->portrefs[i] = (lide_c_fifo_pointer*)context->in[i];
    }
    context->portrefs[context->port_count-1] = (lide_c_fifo_pointer*)context->out1;
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);    
    lide_c_actor_add_portrefs_array((lide_c_actor_context_type *)context, 
            (lide_c_fifo_pointer *)context->in, context->input_port_count);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out1);     
    
    
    return context;
}

/* by default: there is only one input port not empty if actor is enabled*/
boolean lide_c_sca_endcase_enable(lide_c_sca_endcase_context_type *context) {
    boolean result = FALSE;
    int i, tmp;
    tmp = FALSE;
    switch (context->mode) {
    case LIDE_C_SCA_ENDCASE_MODE_PROCESS:
        for(i = 0; i<context->input_port_count; i++){
            if(lide_c_fifo_unit_size_population(context->in[i]) >=1){
                tmp = TRUE;
                context->pindex = i;
                break;
            }
        }
        result = (lide_c_fifo_unit_size_population(context->out1) < 
                lide_c_fifo_unit_size_capacity(context->out1)) && tmp;
                
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.
*******************************************************************************/
void lide_c_sca_endcase_invoke(lide_c_sca_endcase_context_type *context) {

    switch (context->mode) {
    case LIDE_C_SCA_ENDCASE_MODE_PROCESS:
        lide_c_fifo_unit_size_read_advance(context->in[context->pindex]);
        lide_c_fifo_unit_size_write_advance(context->out1);
        context->last_fifo = context->out1;
        
        context->mode = LIDE_C_SCA_ENDCASE_MODE_PROCESS;

        break;
    default:
        context->mode = LIDE_C_SCA_ENDCASE_MODE_ERROR;
        break;
    }
}

void lide_c_sca_endcase_terminate(lide_c_sca_endcase_context_type *context) {
    free(context);
}

/* Set and get for input_port_count function*/
void lide_c_sca_endcase_set_input_port_count(
    lide_c_sca_endcase_context_type *context, 
    int input_port_count){
    context->input_port_count = input_port_count;
    if(context->input_port_count < 1){
        context->mode = LIDE_C_SCA_ENDCASE_MODE_ERROR;
    }else{
        context->mode = LIDE_C_SCA_ENDCASE_MODE_PROCESS;
    }
    return;
}

int lide_c_sca_endcase_get_input_port_count(
    lide_c_sca_endcase_context_type *context){
    return context->input_port_count;
}

/* Set and get function for pointer to input FIFO array*/
void lide_c_sca_endcase_set_in_pointer(
    lide_c_sca_endcase_context_type *context, 
    lide_c_fifo_unit_size_pointer *in){
    context->in = in;
    return;
}

lide_c_fifo_unit_size_pointer *lide_c_sca_endcase_get_in_pointer(
    lide_c_sca_endcase_context_type *context){
    return context->in;
}

void lide_c_sca_endcase_set_index(
lide_c_sca_endcase_context_type *context, int index){
    context->index = index;
    return;
} 

int lide_c_sca_endcase_get_index(
lide_c_sca_endcase_context_type *context){
    return context->index;
} 

void lide_c_sca_endcase_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;
    int i; 
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    
    for(i = 0; i<context->port_count-1; i++){
        port_index = i;
        lide_c_graph_add_connection(graph, 
                                    (lide_c_actor_context_type *)context, 
                                    port_index, direction);
    }
    
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = context->port_count - 1;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
}
