/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_sca_if.h"
#include "lide_c_graph.h"
#include "lide_c_util.h"

#define MAX_FIFO_COUNT  3

struct _lide_c_sca_if_context_struct {
#include "lide_c_sca_context_type_common.h" 


/* FIFO ports. */
/*input*/
lide_c_fifo_unit_size_pointer in1; 
/*    
    pointer to SCA IF actor's input FIFOs array; number of elements in the 
    array is input_port_count; data type of element in the array is 
    lide_c_fifo_unit_size_pointer
*/
lide_c_fifo_unit_size_pointer *out; 


/* number of output ports in the actor*/
int output_port_count;

/* index of output port where output token is produced*/
int pindex;

/* number of fifos in the actor*/
//int fifo_count;

};

/* Comment: output port of if actor is a FIFO array*/
lide_c_sca_if_context_type *lide_c_sca_if_new(
        //lide_c_fifo_unit_size_pointer in1, 
        lide_c_fifo_pointer in1, 
        //lide_c_fifo_unit_size_pointer out1, 
        lide_c_fifo_pointer out1, 
        //lide_c_fifo_unit_size_pointer out2, 
        lide_c_fifo_pointer out2, 
        int output_port_count, int index){
    
    int i;
    lide_c_sca_if_context_type *context = NULL;

    context = (lide_c_sca_if_context_type *)lide_c_util_malloc(
                sizeof(lide_c_sca_if_context_type));
                
    context->mode = LIDE_C_SCA_IF_MODE_PROCESS;
    
    context->enable = (lide_c_actor_enable_ftype)lide_c_sca_if_enable;
    context->invoke = (lide_c_actor_invoke_ftype)lide_c_sca_if_invoke;
    
    context->output_port_count = 2;
    context->out = (lide_c_fifo_unit_size_pointer *)lide_c_util_malloc(
                context->output_port_count * 
                sizeof(lide_c_fifo_unit_size_pointer));
    context->out[0] = (lide_c_fifo_unit_size_pointer)out1;  
    context->out[1] = (lide_c_fifo_unit_size_pointer)out2;
    context->in1 = (lide_c_fifo_unit_size_pointer)in1;
    context->last_fifo = NULL;
    context->index = index;  
    context->pindex = 0;  
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)
                        lide_c_sca_if_connect;
    context->max_port_count = MAX_FIFO_COUNT;           
    /* Initialize portrefs*/
    /*
    context->port_count = context->output_port_count + 1;
    context->portrefs = (lide_c_fifo_pointer **)lide_c_util_malloc(
                        sizeof(lide_c_fifo_pointer *) * context->port_count);
    
    context->portrefs[0] = (lide_c_fifo_pointer*)context->in1;
    for(i = 0; i<context->output_port_count; i++){
        context->portrefs[i+1] = (lide_c_fifo_pointer*)context->out[i];
    }
    */
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in1);  
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out[0]);                                 
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out[1]);
    return context;
}


boolean lide_c_sca_if_enable(lide_c_sca_if_context_type *context) {
    boolean result = FALSE;
    int i, tmp;
    tmp = FALSE;
    switch (context->mode) {
    case LIDE_C_SCA_IF_MODE_PROCESS:
        for(i = 0; i<context->output_port_count; i++){
            if((lide_c_fifo_unit_size_population(context->out[i]) < 
                lide_c_fifo_unit_size_capacity(context->out[i]))){
                tmp = TRUE;
                break;
            }
        }
        result = (lide_c_fifo_unit_size_population(context->in1) >= 1) && tmp;
                
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.
*******************************************************************************/
void lide_c_sca_if_invoke(lide_c_sca_if_context_type *context) {

    switch (context->mode) {
    case LIDE_C_SCA_IF_MODE_PROCESS:
        lide_c_fifo_unit_size_read(context->in1, &context->pindex);
        if(context->pindex >= context->output_port_count){
            fprintf(stderr, "invalid input token in if actor\n");
            context->mode = LIDE_C_SCA_IF_MODE_ERROR;
            break;
        }
        if(lide_c_fifo_unit_size_population(context->out[context->pindex]) < 
                lide_c_fifo_unit_size_capacity(context->out[context->pindex])){
            lide_c_fifo_unit_size_write_advance(context->out[context->pindex]);
            context->last_fifo = context->out[context->pindex];
        }else{
            lide_c_fifo_unit_size_write(context->in1, &context->pindex);
            context->last_fifo = context->in1;
        }
        context->mode = LIDE_C_SCA_IF_MODE_PROCESS;
        break;
 
    default:
        context->mode = LIDE_C_SCA_IF_MODE_ERROR;
        break;
    }
}

void lide_c_sca_if_terminate(lide_c_sca_if_context_type *context) {
    free(context);
}

lide_c_fifo_unit_size_pointer lide_c_sca_if_last_dsg_fifo_output(
        lide_c_sca_if_context_type *context){
    return context->last_fifo;
}


void lide_c_sca_if_set_index(
lide_c_sca_if_context_type *context, int index){
    context->index = index;
    return;
} 

int lide_c_sca_if_get_index(
lide_c_sca_if_context_type *context){
    return context->index;
} 

void lide_c_sca_if_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;
    int i; 
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, 
                                (lide_c_actor_context_type *)context, 
                                port_index, direction);
    
    
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    for(i = 1; i<context->port_count; i++){
        port_index = i;
        lide_c_graph_add_connection(graph, 
                                    (lide_c_actor_context_type *)context, 
                                    port_index, direction);
    }
}