/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lide_c_sca_sch_loop.h"
#include "lide_c_util.h"
#include "lide_c_graph.h"

#define MAX_FIFO_COUNT 2

struct _lide_c_sca_sch_loop_context_struct {
#include "lide_c_sca_context_type_common.h" 

/* sub mode*/
int process_submode;

/* FIFO ports. */
lide_c_fifo_unit_size_pointer in1; /*loop_start*/
lide_c_fifo_unit_size_pointer out1; /*iteration_start*/

/* Number of rest iterations */
int current_iteration;
/* Total number of iterations in the loop*/
int iteration_count;

};

lide_c_sca_sch_loop_context_type *lide_c_sca_sch_loop_new(
        //lide_c_fifo_unit_size_pointer in1, 
        lide_c_fifo_pointer in1, 
        //lide_c_fifo_unit_size_pointer out1, 
        lide_c_fifo_pointer out1, 
        int iteration_count, int index){

    lide_c_sca_sch_loop_context_type *context = NULL;

    context = (lide_c_sca_sch_loop_context_type *)lide_c_util_malloc(
                sizeof(lide_c_sca_sch_loop_context_type));
                
    context->mode = LIDE_C_SCA_SCH_LOOP_MODE_PROCESS;
    context->process_submode = LIDE_C_SCA_SCH_LOOP_SUBMODE_PROCESS_A;
    
    context->enable = (lide_c_actor_enable_ftype)lide_c_sca_sch_loop_enable;
    context->invoke = (lide_c_actor_invoke_ftype)lide_c_sca_sch_loop_invoke;
  
  
    context->in1 = (lide_c_fifo_unit_size_pointer)in1;
    context->out1 = (lide_c_fifo_unit_size_pointer)out1;
    context->last_fifo = NULL;
    
    context->index = index;
    context->subgraphs = NULL;
    context->subgraph_count = 0;
    context->connect = (lide_c_actor_connect_ftype)
                        lide_c_sca_sch_loop_connect;
    context->max_port_count = MAX_FIFO_COUNT;
    /* Initialize portrefs*/
    /*
    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context, 
        FIFO_COUNT, &context->in1, &context->out1);
    */

    lide_c_actor_init_portrefs((lide_c_actor_context_type *)context);
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->in1);    
    lide_c_actor_add_portrefs((lide_c_actor_context_type *)context, 
                                (lide_c_fifo_pointer *)&context->out1); 
    
    context->iteration_count = iteration_count;
    context->current_iteration = context->iteration_count;

    /* iter_num is positive */
    if(context->iteration_count < 1){
        context->mode = LIDE_C_SCA_SCH_LOOP_MODE_ERROR;
    }
    return context;
}


boolean lide_c_sca_sch_loop_enable(lide_c_sca_sch_loop_context_type *context) {
    boolean result = FALSE;

    switch (context->mode) {
    case LIDE_C_SCA_SCH_LOOP_MODE_PROCESS:
        result = (lide_c_fifo_unit_size_population(context->in1) >= 1) &&
                (lide_c_fifo_unit_size_population(context->out1) < 
                lide_c_fifo_unit_size_capacity(context->out1));
        break;
    default:
        result = FALSE;
        break;
    }
    return result;
}

/*******************************************************************************
The invoke method assumes that a corresponding call to enable has
returned TRUE so that the required data is available.
*******************************************************************************/
void lide_c_sca_sch_loop_invoke(lide_c_sca_sch_loop_context_type *context) {

    switch (context->mode) {
    case LIDE_C_SCA_SCH_LOOP_MODE_PROCESS:
        lide_c_fifo_unit_size_read_advance(context->in1);
        
        
           // printf("sca_sch_loop\n");
        if(context->current_iteration > 0){
            context->current_iteration = context->current_iteration-1;
            context->process_submode = LIDE_C_SCA_SCH_LOOP_SUBMODE_PROCESS_A;
        }else{
            context->process_submode = LIDE_C_SCA_SCH_LOOP_SUBMODE_PROCESS_B;
        }
        
        if(context->process_submode == LIDE_C_SCA_SCH_LOOP_SUBMODE_PROCESS_A){
            lide_c_fifo_unit_size_write_advance(context->out1);
            context->last_fifo = context->out1;
            context->mode = LIDE_C_SCA_SCH_LOOP_MODE_PROCESS;  
        }else{
            context->mode = LIDE_C_SCA_SCH_LOOP_MODE_END;
            context->last_fifo = NULL;
        }
        break;
    case LIDE_C_SCA_SCH_LOOP_MODE_END:
        context->mode = LIDE_C_SCA_SCH_LOOP_MODE_END;
        break;
    default:
        context->mode = LIDE_C_SCA_SCH_LOOP_MODE_ERROR;
        break;
    }
}

void lide_c_sca_sch_loop_terminate(
    lide_c_sca_sch_loop_context_type *context){
    free(context);
}

void lide_c_sca_sch_loop_set_current_iteration(
    lide_c_sca_sch_loop_context_type *context, 
    int current_iteration){
    context->current_iteration = current_iteration;
    return;
}

int lide_c_sca_sch_loop_get_current_iteration(
    lide_c_sca_sch_loop_context_type *context){
    return context->current_iteration;
}

void lide_c_sca_sch_loop_set_iteration_count(
    lide_c_sca_sch_loop_context_type *context, 
    int iteration_count){
    context->iteration_count = iteration_count;
    if(context->mode == LIDE_C_SCA_SCH_LOOP_MODE_ERROR){
        context->mode = LIDE_C_SCA_SCH_LOOP_MODE_PROCESS;
    }
        
    if(context->iteration_count < 1){
        context->mode = LIDE_C_SCA_SCH_LOOP_MODE_ERROR;
    }
    return;
}

int lide_c_sca_sch_loop_get_iteration_count(
    lide_c_sca_sch_loop_context_type *context){
    return context->iteration_count;
}

void lide_c_sca_sch_loop_set_index(
lide_c_sca_sch_loop_context_type *context, int index){
    context->index = index;
    return;
} 

int lide_c_sca_sch_loop_get_index(
lide_c_sca_sch_loop_context_type *context){
    return context->index;
} 

void lide_c_sca_sch_loop_connect(lide_c_actor_context_type *context, 
        struct lide_c_graph_context_struct *graph){
    int port_index;
    int direction;
    /* input 1*/
    direction = GRAPH_IN_CONN_DIRECTION;
    port_index = 0;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);                                             
    /* output 1*/
    direction = GRAPH_OUT_CONN_DIRECTION;
    port_index = 1;
    lide_c_graph_add_connection(graph, (lide_c_actor_context_type *)context, 
                                port_index, direction);
}
