#ifndef _lide_c_fifo_h
#define _lide_c_fifo_h

/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2020
Maryland DSPCAD Research Group, The University of Maryland at College Park 
All rights reserved.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/
#include "lide_c_basic.h"

struct lide_c_fifo_struct;

/*****************************************************************************
A pointer to a "lide_c_fifo_read", which is a function that reads
data from FIFO.
*****************************************************************************/
typedef void (*lide_c_fifo_read_ftype) 
        (struct lide_c_fifo_struct *fifo, void *data);

        

/*****************************************************************************
A pointer to a "lide_c_fifo_peek", which is a function that peek 
data from FIFO.
*****************************************************************************/
typedef void (*lide_c_fifo_peek_ftype)
        (struct lide_c_fifo_struct *fifo, void *data);


/*****************************************************************************
A pointer to a "lide_c_fifo_read_advance", which is a function that update  
data population without reading data in FIFO.
*****************************************************************************/
typedef void (*lide_c_fifo_read_advance_ftype)
        (struct lide_c_fifo_struct * fifo);    
        
/*****************************************************************************
A pointer to a "lide_c_fifo_read_block", which is a function that reads a 
block of data from FIFO
*****************************************************************************/
typedef void (*lide_c_fifo_read_block_ftype) 
        (struct lide_c_fifo_struct *fifo, void *data, int size);
        
/*****************************************************************************
A pointer to a "lide_c_fifo_write_function", which is a function that writes
data to the FIFO
*****************************************************************************/
typedef void (*lide_c_fifo_write_ftype) 
        (struct lide_c_fifo_struct *fifo, void *data);

/*****************************************************************************
A pointer to a "lide_c_fifo_write_advance", which is a function that writes
data to the FIFO 
*****************************************************************************/
typedef void (*lide_c_fifo_write_advance_ftype)
        (struct lide_c_fifo_struct *fifo);      

        
/*****************************************************************************
A pointer to a "lide_c_fifo_write_block", which is a function that reads a 
block of data to the FIFO
*****************************************************************************/
typedef void (*lide_c_fifo_write_block_ftype) 
        (struct lide_c_fifo_struct *fifo, void *data, int size);   


/*****************************************************************************
A pointer to a "lide_c_fifo_population", which is a function that gets the 
population of the FIFO
*****************************************************************************/
typedef int (*lide_c_fifo_population_ftype) 
        (struct lide_c_fifo_struct *fifo);   


/*****************************************************************************
A pointer to a "lide_c_fifo_capacity", which is a function that gets the 
capacity of the FIFO
*****************************************************************************/
typedef int (*lide_c_fifo_capacity_ftype) 
        (struct lide_c_fifo_struct *fifo);   
        
/*****************************************************************************
A pointer to a "lide_c_fifo_token_size", which is a function that gets the 
token size
*****************************************************************************/
typedef int (*lide_c_fifo_token_size_ftype) 
        (struct lide_c_fifo_struct *fifo);  
        
/*****************************************************************************
A pointer to a "lide_c_fifo_reset", which is a function that resets the FIFO
*****************************************************************************/
typedef void (*lide_c_fifo_reset_ftype) 
        (struct lide_c_fifo_struct *fifo);  

/*****************************************************************************
A pointer to a "lide_c_fifo_set_index", which is a function that set index of 
FIFO in the dataflow graph
*****************************************************************************/
typedef void (*lide_c_fifo_set_index_ftype) 
        (struct lide_c_fifo_struct *fifo, int index);  

/*****************************************************************************
A pointer to a "lide_c_fifo_get_index", which is a function that get index of 
FIFO in the dataflow graph
*****************************************************************************/
typedef int (*lide_c_fifo_get_index_ftype) 
        (struct lide_c_fifo_struct *fifo);  
        
typedef struct lide_c_fifo_struct {
#include "lide_c_fifo_type_common.h"
} lide_c_fifo_type;

/* A pointer to a basic_fifo. */
typedef lide_c_fifo_type *lide_c_fifo_pointer;

#endif
