/*******************************************************************************
 @ddblock_begin copyright
 
 Copyright (c) 1997-2018
 Maryland DSPCAD Research Group, The University of Maryland at College Park
 
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.
 
 IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.
 
 @ddblock_end copyright
 *******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lide_c_fork_router_ch.h"
#include "lide_c_util.h"

struct _lide_c_fork_router_ch_context_struct {
#include "lide_c_actor_context_type_common.h"
    int control_value;
    int input_value;
    
    /* Actor interface ports. */
    lide_c_fifo_pointer in;

    lide_c_switch_router_port_pointer out1;
    lide_c_switch_router_context_pointer router_out1;
    lide_c_select_router_port_pointer out2;
    lide_c_select_router_context_pointer router_out2;
    lide_c_select_router_port_pointer out3;
    lide_c_select_router_context_pointer router_out3;
    lide_c_select_router_port_pointer out4;
    lide_c_select_router_context_pointer router_out4;
    lide_c_select_router_port_pointer out5;
    lide_c_select_router_context_pointer router_out5;

};


lide_c_fork_router_ch_context_type *lide_c_fork_router_ch_new(lide_c_fifo_pointer in,
        lide_c_switch_router_port_pointer out1,
        lide_c_select_router_port_pointer out2,
        lide_c_select_router_port_pointer out3,
        lide_c_select_router_port_pointer out4,
        lide_c_select_router_port_pointer out5 ) {
    
    lide_c_fork_router_ch_context_type *context = NULL;
    
    context = lide_c_util_malloc(sizeof(lide_c_fork_router_ch_context_type));
    context->mode = LIDE_C_FORK_ROUTER_CH_MODE_ACTIVE;
    context->enable = (lide_c_actor_enable_ftype)lide_c_fork_router_ch_enable;
    context->invoke = (lide_c_actor_invoke_ftype)lide_c_fork_router_ch_invoke;
    context->in = in;
    context->out1 = out1;
    context->out2 = out2;
    context->out3 = out3;
    context->out4 = out4;
    context->out5 = out5;
    context->router_out1 = lide_c_switch_router_port_get_switch_router_context
            (context->out1);
    context->router_out2 = lide_c_select_router_port_get_select_router_context
            (context->out2);
    context->router_out3 = lide_c_select_router_port_get_select_router_context
            (context->out3);
    context->router_out4 = lide_c_select_router_port_get_select_router_context
            (context->out4);
    context->router_out5 = lide_c_select_router_port_get_select_router_context
            (context->out5);
    return context;
}

boolean lide_c_fork_router_ch_enable(lide_c_fork_router_ch_context_type *context) {
    boolean result = FALSE;
    
    switch (context->mode) {
        case LIDE_C_FORK_ROUTER_CH_MODE_INACTIVE:
            
            break;
        case LIDE_C_FORK_ROUTER_CH_MODE_ACTIVE:
            result = (lide_c_fifo_population (context->in) >= 1) ;
            break;
        default:
            result = FALSE;
            break;
    }
    return result;
}

void lide_c_fork_router_ch_invoke(lide_c_fork_router_ch_context_type *context) {
    
    int dl;
    int i;
    
    switch (context->mode) {
        
        case LIDE_C_FORK_ROUTER_CH_MODE_INACTIVE:
            context->mode = LIDE_C_FORK_ROUTER_CH_MODE_INACTIVE;
            break;
            
        case LIDE_C_FORK_ROUTER_CH_MODE_ACTIVE:
            lide_c_fifo_read(context->in, &(context->input_value));
            // printf("\n(fork_router_ch)input_value:%d\n",context->input_value);
            lide_c_switch_router_write(context->router_out1,context->out1,
                    &context->input_value);
            
            void *newtoken =NULL;
            for ( i =0; i < 2 ; i++ ) {
            
                newtoken = malloc(sizeof(int));
                memcpy(newtoken, &context->input_value, sizeof(int));
                lide_c_select_router_write (context->router_out2, context->out2,
                        newtoken);
                free(newtoken);
              //  printf("\n (fork_router_ch) Written to select1\n");
                
                newtoken = malloc(sizeof(int));
                memcpy(newtoken, &context->input_value, sizeof(int));
                lide_c_select_router_write (context->router_out3, context->out3,
                        newtoken);
                free(newtoken);
              //  printf("\n (fork_router_ch) Written to select2\n");
                
                newtoken = malloc(sizeof(int));
                memcpy(newtoken, &context->input_value, sizeof(int));
                lide_c_select_router_write (context->router_out4, context->out4,
                        newtoken);
                free(newtoken);
              //  printf("\n (fork_router_ch) Written to select3\n");
                
                newtoken = malloc(sizeof(int));
                memcpy(newtoken, &context->input_value, sizeof(int));
                lide_c_select_router_write (context->router_out5, context->out5,
                        newtoken);
                free(newtoken);
             //   printf("\n (fork_router_ch) Written to select4\n");
                
            }
    
            context->mode = LIDE_C_FORK_ROUTER_CH_MODE_ACTIVE;
            
            break;
            
        default:
            
            context->mode = LIDE_C_FORK_ROUTER_CH_MODE_ACTIVE;
            
            break;
            
    }
}

void lide_c_fork_router_ch_terminate(lide_c_fork_router_ch_context_type *context) {
    free(context);
}

