/****************************************************************************
@ddblock_begin copyright

Copyright (c) 1997-2017
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "lide_c_util.h"
#include "lide_c_select_router.h"



struct _lide_c_select_router_context_struct {
    /* The number of tokens that all the buffers can hold. */
    int capacity;

    /*  Pointer to an array of buffer start pointers; each buffer start 
        pointer points to the first token placeholder in the buffer*/
    void *buffer_start;

    /*  Pointer to an array of buffer end pointers;
        Pointer to the last token placeholder in the buffer. This member
        is here for convenience (it can also be derived from the 
        buffer start, capacity, and token_size). 
    */
    void *buffer_end;

    /*  The number of tokens that are currently in all the buffers of the 
        router. */
    int population;

    /*  Pointer to an array of read pointers;
        Pointer to the next token placeholider to be read from. */
    void **read_pointer;

    /*  Pointer to an array of write pointers;
        Pointer to the next token placeholider to be written into. */
    void *write_pointer;

    /*  The number of bytes in a single token. */
    int token_size;

    /* router port array*/
    lide_c_select_router_port_pointer *ports;

    /* input_ports_count*/
    int input_ports_count;

    /* output_ports_count*/
    int output_ports_count;

    /* port counts*/
    int ports_count;

    /* index of the router in a dataflow graph. It is flexible and set by the 
    designer*/
    int index;
    
    /* enabled buffer id for write operations*/
    int en_buff_id;
    
    /* methods in router*/
    lide_c_select_router_read_ftype read;
    lide_c_select_router_peek_ftype peek;
    lide_c_select_router_read_advance_ftype read_advance;
    lide_c_select_router_read_block_ftype read_block;
    lide_c_select_router_write_ftype write;
    //lide_c_router_write_advance_ftype write_advance;
    lide_c_select_router_write_block_ftype write_block;

    lide_c_select_router_context_population_ftype get_cpop;
    lide_c_select_router_port_population_ftype get_ppop;
    lide_c_select_router_population_ftype get_pop;

    lide_c_select_router_capacity_ftype get_cap;
    lide_c_select_router_context_capacity_ftype get_ccap;
    lide_c_select_router_port_capacity_ftype get_pcap;

    lide_c_select_router_token_size_ftype  get_token_size;
    lide_c_select_router_reset_ftype reset;
    lide_c_select_router_free_ftype free;


    lide_c_select_router_get_select_router_port_with_index_ftype get_port_withidx;
    lide_c_select_router_get_select_router_port_ftype get_port;


    /* The token printing function (if any) associated with this router. */
    lide_c_select_router_f_token_printing print; 
    
    
    /*parameters*/
    int mark; /*whether to read ctr signal*/
    int ctr; /*which channel is selected*/
    
};

struct _lide_c_select_router_port_struct {
    
    /*  Each buffer start pointer points to the first token placeholder 
        in the buffer*/
    void *buffer_start;

    /* Pointer to the last token placeholder in the buffer. This member
       is here for convenience (it can also be derived from the 
       buffer start, capacity, and token_size). 
    */
    void *buffer_end;

    /*  Pointer to the next token placeholider to be read from. */
    void **read_pointer;

    /*  Pointer to the next token placeholider to be written into. */
    void *write_pointer;

    /* label for input or output port group*/
    int group_label;    
    /* index in the group*/
    int group_idx;    
    /* index in the router */    
    int index;   
    /* Port's current "population" for every port*/    
    int population;  
    /* Router's current "capacity" for every port*/    
    int capacity;   
    /* Read pointer for an output port; set to NULL if input port */
    //void read_pointer;  
    /* Router context */
    lide_c_select_router_context_pointer  select_router_context; 
    /* Get context*/
    lide_c_select_router_port_get_select_router_context_ftype get_context;
    lide_c_select_router_port_population_ftype get_port_pop;
    lide_c_select_router_port_get_index_ftype get_port_index;
    /*parameter*/
    int p_ctr; /*port control signal*/
    
};





/*****************************************************************************
Public functions.
*****************************************************************************/

/* select_router port construct function */
/*  input port label:INPUT_PORT --> 0; 
    output port label: OUTPUT_PORT --> 1 */
lide_c_select_router_port_pointer lide_c_select_router_port_new(
                        lide_c_select_router_context_pointer select_router, 
                        int group_label, int group_idx, 
                        int index, int ctr){
    lide_c_select_router_port_type *port = NULL;

    port = lide_c_util_malloc(sizeof(lide_c_select_router_port_type));
    port->group_label = group_label;
    port->group_idx = group_idx;
    port->population = 0;
    port->capacity = select_router->capacity;
    port->index = index + select_router->input_ports_count * port->group_label + 
                    port->group_idx;
    port->select_router_context = select_router;
    port->get_context = lide_c_select_router_port_get_select_router_context;
    port->get_port_pop = lide_c_select_router_port_population;
    port->get_port_index = lide_c_select_router_port_get_index;
    port->p_ctr = ctr;
    return port;
}

lide_c_select_router_context_pointer lide_c_select_router_context_new(int capacity, 
                                                        int token_size,
                                                        int input_ports_count,
                                                        int output_ports_count,
                                                        int index){
    int i;
    int tmp;
    
    lide_c_select_router_context_pointer context = NULL;
    
    if (sizeof(char) != 1) {
        fprintf(stderr, "incompatible target platform\n");
    }

    if (capacity < 1) {
        return NULL;
    }
    if (token_size <= 0) {
        return NULL;
    }

    context = (lide_c_select_router_context_pointer)lide_c_util_malloc(
                sizeof(lide_c_select_router_context_type));
    context->population = 0;
    context->capacity = capacity;
    context->token_size = token_size;
    context->input_ports_count = input_ports_count;
    context->output_ports_count = output_ports_count;
    context->ports_count = context->input_ports_count + 
                            context->output_ports_count;
    context->index = index;
    context->mark = 1;/*read ctr signal*/
    context->ctr = -1; /*port not selected yet*/
    /* Memory initialization*/
    context->buffer_start = NULL;
    context->buffer_start = lide_c_util_malloc((size_t)(context->capacity * 
                                                context->token_size));
    if (context->buffer_start == NULL){
        free(context);
        return NULL;
    }
    context->buffer_end = ((char*)context->buffer_start) + 
            ((context->capacity - 1) * context->token_size);
    
    /*Read_pointer initialization*/
    context->read_pointer = NULL;
    /* all ports would read data in the select_router*/
    context->read_pointer = lide_c_util_malloc(sizeof(void *) * 
                            context->output_ports_count);
    //context->read_pointer = lide_c_util_malloc(sizeof(void *) * 
    //                        context->ports_count);
    
    /*port initialization*/
    context->ports = NULL;
    context->ports = lide_c_util_malloc(sizeof(lide_c_select_router_port_pointer)*
                                        context->ports_count);
    /* input port initialization*/
    /*control port*/
    i = 0;
    context->ports[i] = lide_c_select_router_port_new(context, INPUT_PORT, i, 
                                                context->index, 1);
    for (i = 1; i < context->input_ports_count; i++){
        context->ports[i] = lide_c_select_router_port_new(context, INPUT_PORT, i, 
                                                context->index, 0);
    }
    /* output port initialization*/
    for (i = context->input_ports_count; i < context->ports_count; i++){
        tmp = i - context->input_ports_count;
        context->ports[i] = lide_c_select_router_port_new(context, OUTPUT_PORT, tmp, 
                                                context->index, 0);
    }

    /* methods */
    context->print = NULL;
    context->read = lide_c_select_router_read;
    context->peek = lide_c_select_router_peek;
    context->read_advance = lide_c_select_router_read_advance;
    context->read_block = lide_c_select_router_read_block;
    context->write = lide_c_select_router_write;
    context->write_block = lide_c_select_router_write_block;
    context->get_pop = lide_c_select_router_population;
    //context->get_port_pop = lide_c_select_router_port_population;
    context->get_cap = lide_c_select_router_capacity;
    context->get_token_size = lide_c_select_router_token_size;
    context->reset = lide_c_select_router_reset;
    context->free = lide_c_select_router_free;
    //context->get_port_index = lide_c_select_router_port_get_index;
    context->get_port_withidx = lide_c_select_router_port_get_select_router_port_pointer;
    context->get_port = lide_c_select_router_get_port;
    //context->get_context = lide_c_select_router_port_get_select_router_context;
    
    context->reset(context);
    return context;
}


void lide_c_select_router_reset(lide_c_select_router_context_pointer context){
    int i = 0;
    context->population = 0;
    context->write_pointer = context->buffer_start;
    context->mark = 1;
    context->ctr = -1;
    //for (i = 0; i<context->ports_count; i++){
    for (i = 0; i<context->ports_count - context->input_ports_count; i++){
        context->read_pointer[i] = context->buffer_start;
    }
    return;
}

void lide_c_select_router_set_token_printing(lide_c_select_router_context_pointer select_router,
        lide_c_select_router_f_token_printing f) {
    select_router->print = f;
    return;
}

void lide_c_select_router_free(lide_c_select_router_context_pointer select_router) {
    int i;
    if (select_router == NULL) {
        return;
    }
    if (select_router->buffer_start != NULL) {
        free(select_router->buffer_start);
    }
    if (select_router->read_pointer != NULL) {
        free(select_router->read_pointer);
    }

    if (select_router->ports != NULL) {
        for (i = 0; i < select_router->ports_count; i++){
            if(select_router->ports[i] != NULL)
                free(select_router->ports[i]);
        }
        free(select_router->ports);
    }   
    free(select_router);
    return;
}


int lide_c_select_router_population(lide_c_select_router_context_pointer select_router) {
    return select_router->population;
}

int lide_c_select_router_capacity(lide_c_select_router_context_pointer select_router) {
    return select_router->capacity;
}

int lide_c_select_router_token_size(lide_c_select_router_context_pointer select_router) {
    return select_router->token_size;
}


void lide_c_select_router_read(lide_c_select_router_port_pointer port, void *data){
    
    int index;
    int i,pop;
    lide_c_select_router_context_pointer select_router;

    select_router = port->select_router_context;
    
    /*check whether it is an output port*/
    if (port->group_label == 0) {
        fprintf(stderr, "select_router read err: It is not output port in the \
                        select_router.\n");
        return;
    }  
    
    /* Make sure the select_router's related port is not empty. */
    if (select_router->population == 0 || port->population == 0) {
        fprintf(stderr, "select_router read err: population in the select_router is 0.\n");
        return;
    }

    /* index in the ports*/
    index = port->group_idx;
    /* The following assumes that sizeof(char) = 1. */
    memcpy(data, select_router->read_pointer[index], select_router->token_size);
    /* Advance the read pointer, using a circular buffering convention. */
    if (select_router->read_pointer[index] == select_router->buffer_end) {
        select_router->read_pointer[index] = select_router->buffer_start;
    } else {
        select_router->read_pointer[index] = 
                ((char*)select_router->read_pointer[index]) + select_router->token_size;
    }

    /* Update the token count. */
    port->population--; 
    pop = 0;
    for(i = select_router->input_ports_count; i < select_router->ports_count; i++){
        if (select_router->ports[i]->population > pop){
            pop = select_router->ports[i]->population;
        }
    }
    if(select_router->population != pop){
        select_router->population = pop;
        for(i = 0; i < select_router->input_ports_count; i++){
            select_router->ports[i]->population = pop;
        }
    }
    return;
}

void lide_c_select_router_read_block(lide_c_select_router_port_pointer port, void *data, 
                            int size){
    int index;
    int i,pop;
    /* Room (number of token spaces) "in front of the read pointer" within 
       the buffer. 
    */
    int room_in_front = 0;

    /* Breakdown in bytes of the amounts of data to read before and after,
       respectively, the read pointer wraps around the end of the buffer.
    */
    int part1_size = 0;
    int part2_size = 0;

    /* A cache of the FIFO token size. */
    int token_size = 0;
    
    lide_c_select_router_context_pointer select_router;
    select_router = port->select_router_context;
    
    /*check whether it is an output port*/
    if (port->group_label == 0) {
        fprintf(stderr, "select_router read err: It is not output port.\n");
        return;
    } 
    
    /* Make sure the select_router's related port is not empty. */
    if (select_router->population < size || port->population < size) {
        fprintf(stderr, "select_router read err: population in the select_router is far \
                        from enough.\n");
        return;
    }

     
    
    /* index in the ports*/
    index = port->group_idx;

    token_size = select_router->token_size;
    room_in_front = ((((char*) (select_router->buffer_end)) - 
            ((char*)(select_router->read_pointer[index]))) / token_size) + 1;
    if (room_in_front >= size) {
        part1_size = size * token_size;
        part2_size = 0;
    } else {
        part1_size = room_in_front * token_size;
        part2_size = (size - room_in_front) * token_size; 
    }
    /* The following assumes that sizeof(char) = 1. */
    memcpy(data, select_router->read_pointer[index], part1_size);
    if (part2_size > 0) {
        data = ((char*)data) + part1_size;
        memcpy(data, select_router->buffer_start, part2_size);
        select_router->read_pointer[index] = ((char*)select_router->buffer_start) + 
                                        part2_size;
    } else {
        if (size == room_in_front) {
            select_router->read_pointer[index] = select_router->buffer_start;
        } else {
            select_router->read_pointer[index] = 
                    ((char*)select_router->read_pointer[index]) + part1_size;
        }
    }
    /* Update Population for port and select_router */
    port->population -= size; 
    pop = 0;
    for(i = select_router->input_ports_count; i < select_router->ports_count; i++){
        if (select_router->ports[i]->population > pop){
            pop = select_router->ports[i]->population;
        }
    }
    if(select_router->population != pop){
        select_router->population = pop;
        for(i = 0; i < select_router->input_ports_count; i++){
            select_router->ports[i]->population = pop;
        }
    }
    return;
}

void lide_c_select_router_peek(lide_c_select_router_port_pointer port, void *data){
    
    int index;
    lide_c_select_router_context_pointer select_router;

    select_router = port->select_router_context;
    
    /*check whether it is an output port*/
    if (port->group_label == 0) {
        fprintf(stderr, "select_router read err: It is not output port in the \
                        select_router.\n");
        return;
    }  
    
    /* Make sure the select_router's related port is not empty. */
    if (select_router->population == 0 || port->population == 0) {
        fprintf(stderr, "select_router read err: population in the select_router is 0.\n");
        return;
    }

    /* index in the ports*/
    index = port->group_idx;
    /* The following assumes that sizeof(char) = 1. */
    memcpy(data, select_router->read_pointer[index], select_router->token_size);
    
    return;
}


void lide_c_select_router_read_advance(lide_c_select_router_port_pointer port){
    
    int index;
    int i,pop;
    lide_c_select_router_context_pointer select_router;

    select_router = port->select_router_context;
    
    /*check whether it is an output port*/
    if (port->group_label == 0) {
        fprintf(stderr, "select_router read err: It is not output port in the \
                        select_router.\n");
        return;
    }  
    
    /* Make sure the select_router's related port is not empty. */
    if (select_router->population == 0 || port->population == 0) {
        fprintf(stderr, "select_router read err: population in the select_router is 0.\n");
        return;
    }

    /* index in the ports*/
    index = port->group_idx;
    
    /* Advance the read pointer, using a circular buffering convention. */
    if (select_router->read_pointer[index] == select_router->buffer_end) {
        select_router->read_pointer[index] = select_router->buffer_start;
    } else {
        select_router->read_pointer[index] = 
                ((char*)select_router->read_pointer[index]) + select_router->token_size;
    }

    /* Update the token count. */
    port->population--; 
    pop = 0;
    for(i = select_router->input_ports_count; i < select_router->ports_count; i++){
        if (select_router->ports[i]->population > pop){
            pop = select_router->ports[i]->population;
        }
    }
    if(select_router->population != pop){
        select_router->population = pop;
        for(i = 0; i < select_router->input_ports_count; i++){
            select_router->ports[i]->population = pop;
        }
    }
    return;
}







void lide_c_select_router_write(lide_c_select_router_port_pointer port, void *data){
    int i;
    int check;
    lide_c_select_router_context_pointer select_router;
    select_router = port->select_router_context;
    
    /*check whether it is an input port*/
    if (port->group_label == 1) {
        fprintf(stderr, "select_router read err: It is not input port.\n");
        return;
    } 
    /*check if control signal*/
    if(select_router->mark == 1 && port->p_ctr == 1){
        select_router->mark = 0;
        select_router->ctr = *((int *)data);
        return;
    }
    
    
    check = (select_router->mark == 0) && 
            ((select_router->ctr == 0 && port->group_idx ==1)
            ||(select_router->ctr == 1 && port->group_idx == 2));
    
    
    
    
    /* if (select_router->mark == 1 && port->p_ctr != 1){
        fprintf(stderr, "waiting for control signal\n");
        return;
    }else if(select_router->mark != 1 && port->p_ctr == 1){
        fprintf(stderr, "waiting for data\n");
        return;
    } */
    
    /*check port: mark == 0 && not control port*/
    /* if(select_router->ctr == 0 && port->group_idx !=1){
        fprintf(stderr, "Wrong port and waiting for data\n");
        return;
    }
    
    if(select_router->ctr == 1 && port->group_idx !=2){
        fprintf(stderr, "Wrong port and waiting for data\n");
        return;
    } */
    if(check == TRUE){
        /* Make sure the select_router is not full. */
        if (select_router->population >= select_router->capacity || 
            port->population >= select_router->capacity) {
            return;
        } 
        
         /* The following assumes that sizeof(char) = 1. */
        memcpy(select_router->write_pointer, data, select_router->token_size);
        /* Advance the read pointer, using a circular buffering convention. */
        if (select_router->write_pointer == select_router->buffer_end) {
            select_router->write_pointer = select_router->buffer_start;
        } else {
            select_router->write_pointer = 
                    ((char*)select_router->write_pointer) + select_router->token_size;
        }
        /* Update the token count. */
        select_router->population++;  
        for(i = 0; i<select_router->ports_count; i++){
            select_router->ports[i]->population++;
        }
        select_router->mark = 1;
    }else{
        fprintf(stderr, "Wrong port and waiting for correct data\n");
        return;
    }    
}

void lide_c_select_router_write_block(lide_c_select_router_port_pointer port, void *data, 
                            int size){

    int i;
    void *tmp_data;
    /* Room (number of token spaces) "in front of the write pointer" within 
       the buffer. 
    */
    int room_in_front = 0;

    /* Breakdown in bytes of the amounts of data to write before and after,
       respectively, the write pointer wraps around the end of the buffer.
    */
    int part1_size = 0;
    int part2_size = 0;
    /* A cache of the FIFO token size. */
    int token_size = 0;
    
    lide_c_select_router_context_pointer select_router;
    select_router = port->select_router_context;
    token_size = select_router->token_size;
    
    /*check whether it is an input port*/
    if (port->group_label == 1) {
        fprintf(stderr, "select_router read err: It is not input port.\n");
        return;
    } 
    
    
    
    /*check if control signal*/
    if(select_router->mark == 1) {
        fprintf(stderr, "waiting for control signal\n");
        return;
    }
    /*check port: mark == 0 && not control port*/
    if(select_router->ctr == 0 && port->group_idx !=1){
        fprintf(stderr, "Wrong port and waiting for data\n");
        return;
    }
    
    if(select_router->ctr == 1 && port->group_idx !=2){
        fprintf(stderr, "Wrong port and waiting for data\n");
        return;
    }
    
    
    /* Make sure the select_router is not full. */
    if ((select_router->population + size > select_router->capacity) || 
        (port->population +  size > select_router->capacity)) {
        return;
    } 
    
      
    /* The following assumes that sizeof(char) = 1. */
    room_in_front = ((((char*) (select_router->buffer_end)) - 
            ((char*)(select_router->write_pointer))) / token_size) + 1;

    if (room_in_front >= size) {
        part1_size = size * select_router->token_size;
        part2_size = 0;
    } else {
        part1_size = room_in_front * select_router->token_size;
        part2_size = (size - room_in_front) * select_router->token_size; 
    }
    /* The following assumes that sizeof(char) = 1. */
    memcpy(select_router->write_pointer, data, part1_size);
    
    if (part2_size > 0) {
        tmp_data = ((char*)data) + part1_size;
        memcpy(select_router->buffer_start, tmp_data, part2_size);
        select_router->write_pointer = ((char*)select_router->buffer_start) + part2_size;
    } else {
        if (size == room_in_front) {
            select_router->write_pointer = select_router->buffer_start;
        } else {
            select_router->write_pointer = ((char*)select_router->write_pointer) + part1_size;
        }
    }

    /* Update the token count. */
    select_router->population += size;  
    for(i = 0; i<select_router->ports_count; i++){
        select_router->ports[i]->population += size;
    }    
    select_router->mark = 1;
    
}



lide_c_select_router_port_pointer lide_c_select_router_get_port(
                        lide_c_select_router_context_pointer select_router, 
                        int group_label, int group_idx){
    int port_index;
    port_index = select_router->input_ports_count * group_label + group_idx;
    return select_router->ports[port_index];
}

int lide_c_select_router_port_population(lide_c_select_router_port_pointer port){
    return port->population;
}

lide_c_select_router_context_pointer lide_c_select_router_port_get_select_router_context(
                                            lide_c_select_router_port_pointer port){
    return port->select_router_context;
}

lide_c_select_router_port_pointer lide_c_select_router_port_get_select_router_port_pointer(
                                        lide_c_select_router_context_pointer select_router, 
                                        int index){
    if(index < select_router -> ports_count && index >= 0){
        return select_router->ports[index];
    }else{
        fprintf(stderr, "port index is out of range.\n");
        return NULL;
    }
        
}

int lide_c_select_router_port_get_index(lide_c_select_router_port_pointer port){
    return port->index;
}